/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_Datalogger.c	  Version: 1.00
   
   Author:  SS	  					  Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19
   
   This verison is designed for the ATmega1284P processor:
   		128K Flash, 4K EEPROM, 16K RAM
   
   Processor support:  ATmega644, ATmega644P, ATmega1284P
	
   This program will not run properly if DIV8 fuse bit is set (programmed)
   and JTAG pins are enabled - this is the chip default.
		
   The IMU64X0 Controller is a set of functions that configure IMU64X0 AVR internal
   peripherals for interrupt driven operation.  Custom applications can easily
   be built on top to these basic functions.  Included is a complete data 
   acquistion controller with user preferences loaded into EEPROM.  The Monitor
   communicates with the user via an RS-232 port using the CDP.   The AVR is 
   a smart powerful 8 bit RISC processor that can be easily programmed in C
   with on chip TWI, SPI and analog and digital IO.  Functions are included 
   to read/write data blocks to the on board Serial Flash.  On board sensors
   (if included) include a dual axis 2G accelerometer, temperature and light
   sensor and battery voltage monitor.
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Datalogger.c,v $
 * Revision 1.00  2013/02/22 SS
 *    Support for Zigbee added
 * Revision 0.96  2011/07/06 SS
 *    New support for IMU6410 and port to AVR Studio 4.19
 * Revision 0.95  2011/07/06 SS
 *    New support for DOF10W
 * Revision 0.94  2011/11/05 SS
 *    Fixed data logging routines and added support for 10DOFW
 * Revision 0.93  2010/11/05 DL
 *    Added TWI functions to ADS1100 ADC chip
 * Revision 0.92  2010/06/11 RS
 *    Changed to support ImageCraft ICCAVR Standard V7.20.compiler
 * Revision 0.91  2010/03/04 SS
 *    DC200 Support added
 * Revision 0.90  2009/06/20 SS
 *    Serial Flash data logger support for 3-axis rate gyro added
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>

#include <string.h>
//#include <stdlib.h>
#include <math.h>
#include <stdio.h>
//#include <AVRdef.h>

#include "IMU64X0_Uart.h"
#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Extern.h"

extern delay_time CalculateStartTime(long delaysec);

/**************************************************************************

	Datalogger Functions

**************************************************************************/
char FlashBlankCheck(void)
{
 	 char byte;
	 
//	 Release_SPI();
//	 Init_SPI_Master();
	 FlashChipSelect(LOW);  
	 SPI_StartReadFlashPage(0);
	 byte = SPI_MasterReceive();
	 FlashChipSelect(HIGH); 
//	 Release_SPI(); 
	 if(byte!=0xFF) return YES;
     else return NO;
}

void DisplayAccelerometerRange(unsigned byte)
{
	 if((byte&0x18)==0x00) printf("2G ");
	 else if((byte&0x18)==0x04) printf("2G ");
	 else if((byte&0x18)==0x08) printf("4G ");
	 else if((byte&0x18)==0x0c) printf("16G ");
}

unsigned char Convertdata2log(unsigned char loggeddata)
{
 	 if((loggeddata&0x07)==0x00)  return 'a';
	 else if((loggeddata&0x07)==0x01)  return 'g';
	 else if((loggeddata&0x07)==0x02)  return 'm';
	 else if((loggeddata&0x07)==0x03)  return 'b';
	 else if((loggeddata&0x07)==0x04)  return '9';
	 else if((loggeddata&0x07)==0x05)  return 't';
	 return 'a';
}

float ConvertXgrav(int xaxis)
{
    if(xaxis>xgmid) return (float)(xaxis-xgmid)/(float)(xgmax-xgmid);
	else return (float)(xaxis-xgmid)/(float)(xgmid-xgmin);
}

float ConvertYgrav(int yaxis)
{
	if(yaxis>ygmid) return (float)(yaxis-ygmid)/(float)(ygmax-ygmid);
	else return (float)(yaxis-ygmid)/(float)(ygmid-ygmin);
}
			
float ConvertZgrav(int zaxis)
{
	if(zaxis>zgmid) return (float)(zaxis-zgmid)/(float)(zgmax-zgmid);
	else return (float)(zaxis-zgmid)/(float)(zgmid-zgmin);
}

int DecodeBufferLength(unsigned char loggeddata)
{
	int numbytes=0;

	if(loggeddata&0x01) numbytes += 6;
	if(loggeddata&0x02) numbytes += 6;
	if(loggeddata&0x04) numbytes += 6;
//	if(loggeddata&0x08) numbytes += 4;

	return numbytes;
}

float CalculateAccelScale(unsigned char sensorrange)
{
	float scale=1.0;

	// Set accelerometer range bits
	if(defaultaccelerometerrange==G2MODE)  scale = 8.0;
	else if(defaultaccelerometerrange==G4MODE) scale = 4.0;
	else if(defaultaccelerometerrange==G8MODE) scale = 2.0;
	else if(defaultaccelerometerrange==G16MODE) scale = 1.0;
 
	return scale;
}

float CalculateGyroScale(unsigned char sensorrange)
{
	float scale=1.0;

	// Set gyro range bits
	if(defaultgyrorange==L250DPS) scale = 1.0; 
	else if(defaultgyrorange==L500DPS) scale = 2.0; 
	else if(defaultgyrorange==L2000DPS)	scale = 8.0; 

	return scale;
}

float CalculateMagScale(unsigned char sensorrange)
{
	float scale=1.0;

	// Set magnetometer range bits
	if(defaultmagnetometerrange==HMC088GA)  scale = 0.677;
	else if(defaultmagnetometerrange==HMC130GA)  scale = 1.0;
	else if(defaultmagnetometerrange==HMC190GA)  scale = 1.462;
	else if(defaultmagnetometerrange==HMC250GA)  scale = 1.923;
	else if(defaultmagnetometerrange==HMC400GA)  scale = 3.077;
	else if(defaultmagnetometerrange==HMC470GA)  scale = 3.623;
	else if(defaultmagnetometerrange==HMC560GA)  scale = 4.310;
	else if(defaultmagnetometerrange==HMC810GA)  scale = 6.250;

	return scale;
}

const char dislogmes1[] PROGMEM = "Sensors logged: ";
const char dislogmes2[] PROGMEM = "accelerometer ";
const char dislogmes3[] PROGMEM = "gyro ";
const char dislogmes4[] PROGMEM = "magnetometer ";
const char dislogmes5[] PROGMEM = "barometer ";
const char dislogmes8[] PROGMEM = "unknown selection";

void DisplayData2Log(void)
{
	 putfs(dislogmes1);
	 if(data2log&0x01) putfs(dislogmes2);
	 if(data2log&0x02) putfs(dislogmes3);
	 if(data2log&0x04) putfs(dislogmes4);
	 if(data2log&0x08) putfs(dislogmes5);
	 if(data2log&0x10) putfs(dislogmes8);
	 printf("\n\r");
}	 

//const char loggermes1[] = "\n\rEntering log data to serial flash mode\n\r";
const char loggermes1[] PROGMEM = "\n\rIMU64X0 MEMs Sensor Logging System\n\r";
const char loggermes2[] PROGMEM = "Default log rate is ";
const char loggermes3[] PROGMEM = "Default sensor ranges: ";
const char loggermes4[] PROGMEM = "\n\rCommands:\n\r";
const char loggermes5[] PROGMEM = "  ? - Logging parameters summary\n\r";
const char loggermes6[] PROGMEM = "  x - Enter sensor range\n\r";
const char loggermes7[] PROGMEM = "  s - Start data logging\n\r";
const char loggermes8[] PROGMEM = "  h - Halt data logging\n\r";
const char loggermes9[] PROGMEM = "  e - Erase chip\n\r";
const char loggermes10[] PROGMEM = "  d - Select sensor(s) to log (a-accelerometer,g-gyro,m-mag,b-barometer)\n\r";
const char loggermes11[] PROGMEM = "  r - Retrieve data\n\r";
const char loggermes12[] PROGMEM = "  l - Log rate (msec)\n\r";
const char loggermes13[] PROGMEM = "  f - Output format (raw,processed,minmax)\n\r";
const char loggermes14[] PROGMEM = "  q - Quit data logging mode\n\r";
const char loggermes15[] PROGMEM = "\n\rSelect MinMax Mode [e-Enable,d-Disable,]: ";
const char loggermes16[] PROGMEM = "\n\rLogging mode started";
const char loggermes17[] PROGMEM = "\n\rSelect sensor(s) to log [a-accelerometer,g-gyro,m-mag,b-baro]: ";
const char loggermes18[] PROGMEM = "\n\rLogging halted";
const char loggermes19[] PROGMEM = "\n\rEnter new log rate (msec): ";
const char loggermes21[] PROGMEM = "\n\rRetreive logged data\n\r";
const char loggermes22[] PROGMEM = "\n\rSelect output format [r-raw,p-processed,m-minmax]: ";
const char loggermes35[] PROGMEM = "\n\rDefault termination mode is ";
const char loggermes51[] PROGMEM = "  p - Powerup log start mode\n\r";
const char loggermes52[] PROGMEM = "  a - Arm/disarm switch PB0\n\r";
const char loggermes55[] PROGMEM = "  t - Terminate mode (b,t,h)\n\r";
const char loggermes57[] PROGMEM = "  w - Start logging at future time\n\r";
const char loggermes58[] PROGMEM = "  u - Store data in uSD or Serial Flash\n\r";
const char loggermes59[] PROGMEM = "  v - Display logging output\n\r";
const char loggermes61[] PROGMEM = "\n\rSensor Range Set Menu\n\r";
const char loggermes62[] PROGMEM = "  Select sensor [a-accel,g-gyro,m-mag or e-exit]: \n\rr>";
const char loggermes64[] PROGMEM = "\n\r  Set accelerometer range [2-2g,4-4g,8-8g or 6-16g]: ";
const char loggermes65[] PROGMEM = "\n\r  Set gyro range [2-250dps,5-500dps or 9-2000dps]: ";
const char loggermes66[] PROGMEM = "\n\r  Set magnetometer range [0-0.88mg,1-1.3mg,2-1.7mg,6-5.6mg or 7-8.1mg]: ";
const char loggermes85[] PROGMEM = "  i - Enter Sensor ID1 or ID2 name\n\r";
const char loggermes135[] PROGMEM = "  c - Display command summary\n\r";
const char loggermes145[] PROGMEM = "\n\rEnter termination mode [b-switch,t-time,h-halt]: ";
const char loggermes193[] PROGMEM = "\n\rSelect Sensor ID 1 or 2: ";
const char loggermes195[] PROGMEM = "\n\rEnter new Sensor ID name [20 chars]: ";
const char loggermes571[] PROGMEM = "  j - Threshold mode setup\n\r";
const char loggermes572[] PROGMEM = "  y - Log restart mode setup\n\r";
const char loggermes573[] PROGMEM = "  m - Enable/Disable printing MinMax values at end of file\n\r";

const char sensorrangemes1[] PROGMEM = "accel - ";
const char sensorrangemes2[] PROGMEM = "gyro -  ";
const char sensorrangemes3[] PROGMEM = "mag -  ";

void DisplaySensorRanges()
{
	 putfs(loggermes3);

	 // Display accelerometer range
	 if(defaultaccelerometerrange=='6') putchar('1');
	 putchar(defaultaccelerometerrange);
	 printf("g ");

	 // Display gyro range
	 if(defaultgyrorange==L250DPS) printf("250dps "); 
	 else if(defaultgyrorange==L500DPS) printf("500dps ");
	 else if(defaultgyrorange==L2000DPS) printf("2000dps ");

	 // Display magnetometer range
	if(defaultmagnetometerrange==HMC088GA)  printf("0.88mg"); 
	else if(defaultmagnetometerrange==HMC130GA)  printf("1.30mg"); 
	else if(defaultmagnetometerrange==HMC190GA)  printf("1.90mg"); 
	else if(defaultmagnetometerrange==HMC250GA)  printf("2.50mg"); 
	else if(defaultmagnetometerrange==HMC400GA)  printf("4.00mg"); 
	else if(defaultmagnetometerrange==HMC470GA)  printf("4.70mg"); 
	else if(defaultmagnetometerrange==HMC560GA)  printf("5.60mg"); 
	else if(defaultmagnetometerrange==HMC810GA)  printf("8.10mg"); 	
}

// Retreive logged flash data until no more left.
// The retreival prints the time stamp
// then outputs 10DOF mems data.
// Data structure is as follows:
//    Data present in page - 'y' else anything else
//    Time stamp - day,hour,min,second,msec 
void RetreiveFlashData(void)
{
     unsigned char notfinished,msec,second,min,hour,day,loggeddata;
 	 unsigned char i,byte;
	 unsigned char inchar,sensorrange;
	 unsigned int temperature,pressure;
	 int xaxis=0,yaxis=0,zaxis=0,agyro=0,bgyro=0,cgyro=0,pagecnt,lograte,buflength=0;
	 int xmag=0,ymag=0,zmag=0;
	 int xmin,ymin,zmin,xmax,ymax,zmax;
	 int bytesinrecord,headerlength,buflengthend=0;
	 long recordcnt;
	 float floattime;
	 float ftemperature=0.0,fpressure=0.0,gyroscale,magscale;
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	 
	 //Release_SPI();
	 //Init_SPI_Master();
	 //  Set PC4 low - Flash CS
	 FlashChipSelect(LOW);  

	 SPI_StartReadFlashPage(0);
	 pagecnt = 1;
	 recordcnt = 1;
	 notfinished = YES;
	 xmin = ymin = zmin = 1024;
	 xmax = ymax = zmax = 0;

	 while((notfinished==YES)) {
	 	 inchar = getcharnw();
		 // Pause output until any key hit
		 if(inchar=='p') while((getcharnw()==0)) ;
		 // Halt output and exit data log dump
		 else if(inchar!=0) break;

	     byte = SPI_MasterReceive();

		 //printf("\n\rfirst byte = %x\n\r",byte);

		 // See if first page byte is 'y' - process page if it is
		 if(byte=='y') {
		     loggeddata = SPI_MasterReceive();
			 sensorrange = SPI_MasterReceive();

			 //printf("logged data = %c  combinedbyte= %x\n\r",loggeddata,combinedbyte);

		     // Retrieve log rate
			 lograte = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();

		     // Retrieve time stamp
			 day = SPI_MasterReceive();
			 hour = SPI_MasterReceive();
			 min = SPI_MasterReceive();
			 second = SPI_MasterReceive();
			 msec = SPI_MasterReceive();
			 headerlength = 10;

			 // Retrieve barometer data and convert to float equivalent
			 if(loggeddata&0x08) {
			     headerlength = 14;
		    	 u.tempbuf[1] = SPI_MasterReceive();
				 u.tempbuf[0] = SPI_MasterReceive();
				 temperature = u.outint;
				 //printf(" u.outint=%x %x ",u.outint,temperature);
		    	 u.tempbuf[1] = SPI_MasterReceive();
				 u.tempbuf[0] = SPI_MasterReceive();
				 pressure = u.outint;
				 //printf(" u.outint=%x %x ",u.outint,pressure);
				 ftemperature = CalculateBMP085Temperature(temperature);
				 fpressure = CalculateBMP085pressurefloat(temperature,pressure);
			     }

			 if((pagecnt==1)&&(logoutputmode==VERBOSE))  {
			     printf("Sensor ID: %s  Sensor ID2: %s\n\r",deviceidtext,deviceidtext2);
	 			 DisplayData2Log();
				 DisplaySensorRanges();
			     printf("%c %4d msec\n\r",loggeddata,lograte);
				 pagecnt++;
				 }
			 floattime = (float) second + ((float) (msec*4))/1000.0;
			 
			 // Output logged data if at least one sensor is present
			 if(loggeddata) {
			     bytesinrecord = DecodeBufferLength(loggeddata);
	 			 //printf("\n\rbytesinrecord=%d\n\r",bytesinrecord);

			     if(flashtype0==AT45DB081ID) buflength = (256-headerlength)/bytesinrecord;
	 		     else if(flashtype0==AT45DB642ID) buflength = (1024-headerlength)/bytesinrecord;

			     for(i=0; i<buflength; i++) {
				     if(loggeddata&0x01) {
				         xaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
				         yaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
				         zaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 }
					 if(loggeddata&0x02) {
						 agyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 bgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 cgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 }
 					 if(loggeddata&0x04) {
					     xmag = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 ymag = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 zmag =	((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
						 }

				     printf("%ld ",recordcnt++);
			         printf("%02d:%02d:%02d:%#02.3f",day,hour,min,floattime);
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 if(logoutputmode=='r') {
						 if(loggeddata&0x01) printf(" %4d %4d %4d ",xaxis,yaxis,zaxis);
						 if(loggeddata&0x02) printf("%4d %4d %4d ",agyro,bgyro,cgyro);
						 if(loggeddata&0x04) printf("%4d %4d %4d ",xmag,ymag,zmag);
						 if(loggeddata&0x08) printf("%4f %4f",bmptemperature,bmppressure);
						 printf("\n\r");
						 }
					 else {
						 if(loggeddata&0x01) {
						     // If full resolution mode active don't need to scale
						     //accelscale = CalculateAccelScale(sensorrange);
						     printf(" %#5.3f %#5.3f %#5.3f ",
						 		(float)xaxis*0.0039,(float)yaxis*0.0039,
								(float)zaxis*0.0039);
								}
						 if(loggeddata&0x02) {
						     gyroscale = CalculateGyroScale(sensorrange);
						     printf("%#6.1f %#6.1f %#6.1f ",
						        ((float)agyro)*250.0*gyroscale/32768.0,((float)bgyro)*250.0*gyroscale/32768.0,
								((float)cgyro)*250.0*gyroscale/32768.0);
							 }
						 if(loggeddata&0x04) {
						     magscale = CalculateMagScale(sensorrange);
						     printf("%#6.3f %#6.3f %#6.3f ",
						        ((float)xmag)*(1.3*magscale)/2048.0,((float)ymag)*(1.3*magscale)/2048.0,
								((float)zmag)*(1.3*magscale)/2048.0);
							 }
						 if(loggeddata&0x08) printf("%#4.1f %#6.4f",ftemperature,fpressure);
						 printf("\n\r");
					     }
				     }

				 // Skip the blank bytes at the end of the page
			     if(flashtype0==AT45DB081ID) {
				     //buflength = 9;
					 buflengthend = 256 - headerlength - buflength*bytesinrecord;
					 }
	 		     else if(flashtype0==AT45DB642ID) {
				     //buflength = 3;
					 buflengthend = 1024 - headerlength - buflength*bytesinrecord;
					 }
				 for(i=0; i<buflengthend; i++ )  SPI_MasterReceive();			 		 		 
		         }
			 }
		 else {
		     if(logoutputmode==VERBOSE)  printf("Finished processing pages");
			 notfinished = NO;
//			 xgrav = ConvertXgrav(xmax);
//			 ygrav = ConvertYgrav(ymax);
//			 zgrav = ConvertZgrav(zmax);
//			 printf("\n\rMax G: %# 6.3f %# 6.3f %# 6.3f\n\r",xgrav,ygrav,zgrav);
//			 xgrav = ConvertXgrav(xmin);
//			 ygrav = ConvertYgrav(ymin);
//			 zgrav = ConvertZgrav(zmin);
//			 printf("Min G: %# 6.3f %# 6.3f %# 6.3f\n\r",xgrav,ygrav,zgrav);
			 } 
		 }
     
	 // Set PC4 high - deselect Flash chip
	 FlashChipSelect(HIGH); 
	 //Release_SPI(); 
}

void FlashChipErase(void)
{
// 	 Release_SPI();
//	 Init_SPI_Master();
	 printf("\n\rErase serial flash chip");
	 ChipErase();
//	 Release_SPI();
//	 Init_SPI_Master();
	 WaitForFlashFree();
//	 Release_SPI();
}

// Combine device id into same byte as data logged
unsigned char CombinedeviceID(unsigned char data2log)
{
 	unsigned byte=0;
	
	if(data2log=='a') byte = 0;
	else if(data2log=='g') byte = 1;
	else if(data2log=='m') byte = 2;
	else if(data2log=='b') byte = 3;
	else if(data2log=='9') byte = 4;
	else if(data2log=='t') byte = 5;
	
	// Or with data2log value for recording
	if(deviceid==LEFTARM)  byte |= 0x20;  // left arm
	else if(deviceid==RIGHTARM)  byte |= 0x40;  // right arm
	else if(deviceid==UPPERSPINE)  byte |= 0x60;  // upper spine
	else if(deviceid==LOWERSPINE)  byte |= 0x80;  // lower spine
	else if(deviceid==SLED)  byte |= 0xa0;  // sled

	// Or with accelerometer range setting
	if(defaultaccelerometerrange==G2MODE) byte |= 0x00;
	else if(defaultaccelerometerrange==G4MODE) byte |= 0x08;
	else if(defaultaccelerometerrange==G8MODE) byte |= 0x10;
	else if(defaultaccelerometerrange==G16MODE) byte |= 0x18;
	
    return byte;
}

unsigned char SensorRangeByte()
{
	unsigned char sensorrangebyte=0;
	
	// Set accelerometer range bits
	if(defaultaccelerometerrange==G2MODE)  sensorrangebyte = 0;
	else if(defaultaccelerometerrange==G4MODE) sensorrangebyte = 1;
	else if(defaultaccelerometerrange==G8MODE) sensorrangebyte = 2;
	else if(defaultaccelerometerrange==G16MODE) sensorrangebyte = 3;
 
	// Set gyro range bits
	if(defaultgyrorange==L250DPS) sensorrangebyte |= (0<<2); 
	else if(defaultgyrorange==L500DPS) sensorrangebyte |= (1<<2); 
	else if(defaultgyrorange==L2000DPS)	sensorrangebyte |= (2<<2); 

	// Set magnetometer range bits
	if(defaultmagnetometerrange==HMC088GA)  sensorrangebyte |= (0<<4);
	else if(defaultmagnetometerrange==HMC130GA)  sensorrangebyte |= (1<<4);
	else if(defaultmagnetometerrange==HMC190GA)  sensorrangebyte |= (2<<4);
	else if(defaultmagnetometerrange==HMC250GA)  sensorrangebyte |= (3<<4);
	else if(defaultmagnetometerrange==HMC400GA)  sensorrangebyte |= (4<<4);
	else if(defaultmagnetometerrange==HMC470GA)  sensorrangebyte |= (5<<4);
	else if(defaultmagnetometerrange==HMC560GA)  sensorrangebyte |= (6<<4);
	else if(defaultmagnetometerrange==HMC810GA)  sensorrangebyte |= (7<<4);

	return sensorrangebyte;
}

// Log data scheduler
// Always fill the srambuf before starting the Serial Flash page write
// This keeps the scheduler very simple
void LogData(int logmsec)
{
	 int i,buflength=0,srambufptr=0,bytesinrecord,headerlength;
	 unsigned int pressure;
	 union {
	    unsigned char tempbuf[2];
		int outiint;
		} u;
	 union {
	    unsigned char tempbuf[2];
		unsigned int outiint;
		} uu;

 	 // Setup beginning information for flash page
	 // First byte is always 'y' character indicating flash page is full
	 // Next the sample rate is written followed by the current time
	 // so retrieved data can be properly time stamped.
	 srambuf[srambufptr++] = 'y';
//	 srambuf[srambufptr++] = CombinedeviceID(data2log);
	 srambuf[srambufptr++] = data2log;
	 srambuf[srambufptr++] = SensorRangeByte();
	 u.outiint = logmsec;
	 srambuf[srambufptr++] = u.tempbuf[1];
	 srambuf[srambufptr++] = u.tempbuf[0];
	 srambuf[srambufptr++] = days;
	 srambuf[srambufptr++] = hours; 
	 srambuf[srambufptr++] = minutes;
	 srambuf[srambufptr++] = seconds;
	 srambuf[srambufptr++] = rtcclockcnt;
	 headerlength = 13;

	 // Get the barometer and temperature reading
	 // This is only written once per flash page write
	 if(data2log&0x08) {
	     headerlength = 17;
		 Check_BMP085_Update_Int();
		 uu.outiint = ibmptemperature;
		 srambuf[srambufptr++] = uu.tempbuf[1];
		 srambuf[srambufptr++] = uu.tempbuf[0];	 	 
		 pressure = Finish_Read_BMP085_Pressure();
		 uu.outiint = ibmppressure;
		 srambuf[srambufptr++] = uu.tempbuf[1];
		 srambuf[srambufptr++] = uu.tempbuf[0];
		 }

	 // Now see if this is an AT45DB081 or AT45DB642 Serial Flash
	 // AT45DB081 has 256 byte page while the 642 has 1024 bytes
     bytesinrecord = DecodeBufferLength(data2log);

     if(flashtype0==AT45DB081ID) buflength = (256-headerlength)/bytesinrecord;
	 else if(flashtype0==AT45DB642ID) buflength = (1024-headerlength)/bytesinrecord;

 	 // Setup log rate
	 mseccountdown2 = logmsec;

	 // Store number of bytes based on sensors to be logged
	 for(i=0; i<buflength; i++) {
	     if(data2log&0x01) {
			 u.outiint = SPI_ADXL345_ReadAxis(ADXL345XAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 u.outiint = SPI_ADXL345_ReadAxis(ADXL345YAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 u.outiint = SPI_ADXL345_ReadAxis(ADXL345ZAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 }
		 if(data2log&0x02) {
			 u.outiint = SPI_L3G4200D_ReadAxis(L3G4200XAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
		     u.outiint = SPI_L3G4200D_ReadAxis(L3G4200YAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 u.outiint = SPI_L3G4200D_ReadAxis(L3G4200ZAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 }
		 if(data2log&0x04) {
			 u.outiint = HMC5883_ReadAxis(HMC5883_XAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 u.outiint = HMC5883_ReadAxis(HMC5883_YAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 u.outiint = HMC5883_ReadAxis(HMC5883_ZAXIS);
			 srambuf[srambufptr++] = u.tempbuf[1];
			 srambuf[srambufptr++] = u.tempbuf[0];
			 }
		 // Wait for sample rate to expire
		 while(mseccountdown2) ;
	 	 mseccountdown2 = logmsec;
		 }
	
	 // Serial Flash CS low
	 FlashChipSelect(LOW);	
	  
	 //  Select Buffer to write too
	 if(bufferidptr==1)  SPI_MasterTransmit(SF_WRITE_BUFFER1);
	 else  SPI_MasterTransmit(SF_WRITE_BUFFER2);
	 
	 //printf("ptr=%d ",bufferidptr);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(0);
	 //ConvertPower2FlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);

	 // Transfer contents of srambuf into the Serial Flash Ram buffer
	 srambufptr = 0;
	 if(flashtype0==AT45DB081ID) {
	     for(i=0; i<256; i++)  SPI_MasterTransmit(srambuf[srambufptr++]);
		 }
	 else {
	     for(i=0; i<1024; i++)  SPI_MasterTransmit(srambuf[srambufptr++]);
		 }	     

	 
	 // Serial Flash CS high
	 FlashChipSelect(HIGH);
	 
	 // Store flash buffer to flashpage
	 FlashChipSelect(LOW);
	  
	 // Transfer buffer n to flash memory
	 if(bufferidptr==1) SPI_MasterTransmit(SF_BUF1_TO_FLASH_NOERASE);
	 else SPI_MasterTransmit(SF_BUF2_TO_FLASH_NOERASE);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(flashaddress);
	 //ConvertPower2FlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Progamming the flash can take up to 40msec so make sure the data 
	 // logging rate is not to high
	 FlashChipSelect(HIGH);
	 
	 // Flip the bufferid ptr
	 if(bufferidptr==1) bufferidptr = 2;
	 else bufferidptr = 1;

 	 if(flashtype0==AT45DB081ID)  {
	     flashaddress += 256;
		 if(++pagecnt>4096) loggingmode = HALT;
		 }
	 else if(flashtype0==AT45DB642ID)  {
	     flashaddress += 1024;
	 	 if(++pagecnt>8192)  loggingmode = HALT;
		 }

	 //Release_SPI();
}

unsigned char ConvertMagValue(char inchar)
{
	if(inchar=='0')  return 0;
	if(inchar=='1')  return (1<<5);
	if(inchar=='2')  return (2<<5);
	if(inchar=='3')  return (3<<5);
	if(inchar=='4')  return (4<<5);
	if(inchar=='5')  return (5<<5);
	if(inchar=='6')  return (6<<5);
	if(inchar=='7')  return (7<<5);
	return (1<<5);
}

//  For the moment log everything
void DisplayCommandSummary(void)
{
	 putfs(loggermes4);
	 putfs(loggermes5);
	 putfs(loggermes51);
	 putfs(loggermes52);
	 putfs(loggermes55);
	 putfs(loggermes57);
	 putfs(loggermes571);
	 putfs(loggermes572);
	 putfs(loggermes573);
	 putfs(loggermes58);
	 putfs(loggermes59);
	 putfs(loggermes6);
	 putfs(loggermes7);
	 putfs(loggermes8);
	 putfs(loggermes85);
	 putfs(loggermes9);
	 putfs(loggermes10);
	 putfs(loggermes11);
	 putfs(loggermes12);
	 putfs(loggermes13);
	 putfs(loggermes135);
	 putfs(loggermes14);
}

void SensorRangeMenu()
{
    char inchar;

	putfs(loggermes61);	
	putfs(loggermes62);
	putchar(inchar = getchar());
	while(inchar!='e') {
		if(inchar=='a') {
			putfs(loggermes64);
		    defaultaccelerometerrange = getchar();
		    putchar(defaultaccelerometerrange);
			ADXL345_SetRange(defaultaccelerometerrange);
		    }
		else if(inchar=='g') {
			putfs(loggermes65);
		    defaultgyrorange = getchar();
		    putchar(defaultgyrorange);
			Set_L3G4200D_Range(defaultgyrorange);
		    }
		else if(inchar=='m') {
			putfs(loggermes66);
		    inchar = getchar();
		    putchar(inchar);
			defaultmagnetometerrange = ConvertMagValue(inchar);
			Set_HMC5883_Range(defaultmagnetometerrange);
		    }
        printf("\n\rr>");
		putchar(inchar = getchar());
		}
}

void ThresholdModeMenu()
{
	char inchar;
	int i;

	inchar = ' ';
	putchar(inchar);
	while(inchar!='e') {
		printf("\n\rThreshold mode setup: [m-mode,p-percentage,s-parameter to check,e-exit]: ");
		inchar = getchar();
		putchar(inchar);
	    if(inchar=='m') {
			if(thresholdmode==ACTIVE)  {
			    printf("\n\rThreshold mode inactive");
				thresholdmode = INACTIVE;
				}
			else {
			    printf("\n\rThreshold mode active");
				thresholdmode = ACTIVE;
				}				
		    }
	    if(inchar=='p') {
			printf("\n\rEnter percentage [xx.x]: ");
			localecho = YES;
			scanf("%f",&thresholdpercent);
			getchar();
			localecho = NO;
			thresholdmode = ACTIVE;
		    }
		else if(inchar=='s') {
			printf("\n\rCurrent selection: ");
			for(i=0; i<9; i++) {
				if(thresholdcheck[i]=='y') printf("y ");
				else printf("n ");
				}
			printf("new: ");
			for(i=0; i<9; i++) {
			    inchar = getchar();
				putchar(inchar);
			    if(inchar=='y') thresholdcheck[i] = 'y';
			    else thresholdcheck[i] = 'n';
				inchar = getchar();  //get rid of space 
				putchar(inchar);
			    }
		    }
		}
}

unsigned char InsertSensorID(char sensorselect)
{
	if(sensorselect=='a') return 0x01;
	else if(sensorselect=='g') return 0x02;
	else if(sensorselect=='m') return 0x04;
	else if(sensorselect=='b') return 0x08;
	return 0;
}

void SelectSensorsToLog()
{
	char inchar;
	int ptr=0;

	data2log = 0;
	putfs(loggermes17);
	while((inchar=getchar())!=CR) {
	    putchar(inchar);
		data2log |= InsertSensorID(inchar);
		if(ptr++>=5) break;
	    }
}

// Display max logging time based on Flash type, log rate and sensors logged
void DisplayMaxLogTime()
{
	int temp=0,bytesinrecord,bytespersec;

	if(data2log&0x80) temp = 4;
	bytesinrecord = DecodeBufferLength(data2log);
	bytespersec = (1000/defaultlograte)*(10 + temp + DecodeBufferLength(data2log));
     
	if(flashtype0==AT45DB081ID) printf("\n\rMaximum log time %ld seconds to serial flash for current sensor mix",((long)4096*(long)256)/(long)bytespersec);
	else if(flashtype0==AT45DB642ID) printf("\n\rMaximum log time %ld seconds to serial flash for current sensor mix",((long)8192*(long)1024)/(long)bytespersec);
}

void DataLoggerSignon()
{
	int hours,minutes,seconds,i;

 	putfs(loggermes1);
	printf("Sensor ID: %s  Sensor ID2: %s\n\r",deviceidtext,deviceidtext2);
 	putfs(loggermes2);
	printf("%d msec per sample\n\r",defaultlograte);
	if(storageselection==SERIALFLASH) printf("Store data in Serial Flash\n\r");
	else {
	    printf("Store data in microSD in file %s\n\r",microsdname);
		printf("File naming mode is ");
		if(usdfilenamemode==INCREMENTAL) {
		    printf("incremental\n\r");
			printf("Current file number is %d\n\r",usdfilenamenumber);
			}
		else printf("unique\n\r");
		}
	DisplayData2Log();
	DisplaySensorRanges();
	putfs(loggermes35);
	if(terminationmode==BUTTON) printf("switch");
	else if(terminationmode==HALT) printf("halt");
	else {
		hours = timeoutperiod/3600;
		if(timeoutperiod>3600) printf("%d hours, ",hours);
		minutes = (timeoutperiod-hours*3600)/60;
		if(timeoutperiod>60) printf("%d minutes, ",minutes);
		seconds = timeoutperiod - (hours*3600) - minutes*60;
	    printf("%d second timeout",seconds);
		}
	if(timedstartmode==YES) 
	       printf("\n\rStart logging at %2d:%2d:%2d\n\r",hourstart,minutestart,secondstart);
	if(armpb0==YES) printf("\n\rLogging start by PB0 enabled");
	else printf("\n\rLogging start by PB0 disabled");
	if(thresholdmode==ACTIVE) {
	    printf("\n\rThreshold mode active - trigger %1.2f percent\n\r",thresholdpercent);
		printf("Threshold detection vector (ax,ay,az,gx,gy,gz,mx,my,mz): ");
		for(i=0; i<9; i++) {
		    if(thresholdcheck[i]=='y') printf("y ");
			else printf("n ");
			}
		}
	else printf("\n\rThreshold mode inactive\n\r");
	printf("Logging starts ");
	if(startloggingmode==ONPOWERUP)  printf("on power up\n\r");
	else printf("not on power up\n\r");
	if(logrestart==YES) {
	    printf("Logging restarts automatically %d times\n\r",logrestartmax);
		printf("Current log restart count is %d",logrestartcnt);
		}
	else printf("Logging does not automatically restart");
}

// Log restart menu
void LogRestartMenu()
{
	char inchar;

	printf("\n\rEnter log restart mode [o-on,f-off]: ");	
	inchar = getchar();
	putchar(inchar);
	if(inchar=='o') {
	    logrestart = YES;
		startloggingmode = ONPOWERUP;
		}
	else {
	    logrestart = NO;
		startloggingmode = NOTONPOWERUP;
		}
	if(logrestart==YES) {
	    printf("\n\rEnter number of log restarts [int]: ");
	    localecho = YES;
		scanf("%d",&logrestartmax);
		localecho = NO;
		getchar();
		logrestartcnt = 0;  // reset log restart tally
		printf("\n\rMaximum number of logging restarts is %d",logrestartmax);
		}
}

// Enable logging start at a time in the future
void SetupTimedStart()
{
	unsigned char gday,ghour,gminute,gsecond;
	delay_time starttime;

    cli();
		gday = days;
		ghour = hours;
		gminute = minutes;
		gsecond = seconds;
	sei();

	printf("\n\rCurrent time is %d %02d:%02d:%02d\n\r",gday,ghour,gminute,gsecond);	
	printf("Enter future start time in seconds from now: ");
	localecho = YES;
	scanf("%d",&delaystarttimesec);
	localecho = NO;
	getchar();
	starttime = CalculateStartTime(delaystarttimesec);
	daystart = starttime.day;
	hourstart = starttime.hour;
	minutestart = starttime.minute;
	secondstart = starttime.second;
	printf("\n\rStart logging at %d %02d:%02d:%02d",daystart,hourstart,minutestart,secondstart);
	timedstartmode = YES;
}

#ifdef IMU6410
const char storagemess1[] PROGMEM = "\n\rNo uSD available - default storage is Serial Flash ";
#else
const char storagemess1[] PROGMEM = "\n\rNo uSD available - IMU6420 does not have Serial Flash ";
#endif

const char storagemess2[] PROGMEM = "\n\rSelect storage [u-uSD, f-Serial Flash]: ";
const char storagemess25[] PROGMEM = "\n\rFile name mode [i-Automatic incremental, u-Unique name]: ";
const char storagemess3[] PROGMEM = "\n\rEnter storage file name (<=12 characters): ";

// Select storage device - uSD or Serial Flash
// Presence of the uSD is detected at power up - if a uSD is not present then 
// it can't be selected.  The Serial Flash maximum size is 8Mx8 with standard size
// 1Mx8
char StorageDeviceMenu()
{
	char inchar;
	int i;

	if(usdpresent==NO) {
		putfs(storagemess1);
#ifdef IMU6410
		storageselection = SERIALFLASH;
#else 
		storageselection = MICROSD;
#endif
		return 0;
	    }
    putfs(storagemess2);
	StorageChoice = unDONE;
	while(StorageChoice!=DONE)  {
		if(Uart1Channel == YES) {
			while(getcharnw1() != ':');
			inchar = getchar1();
			putchar(inchar);
		    }
		else {
			inchar = getchar();
			putchar(inchar);
	 	    }

		if(inchar=='u') {
			storageselection = MICROSD;
			putfs(storagemess25);
			if(Uart1Channel == YES) {
				while(getcharnw1() != ':');
				usdfilenamemode = getchar1();
				putchar(usdfilenamemode);
			    }
			else {
				usdfilenamemode = getchar();
				putchar(usdfilenamemode);
			    }
			if(usdfilenamemode=='i') {
				usdfilenamenumber = 1;
				sprintf(microsdname,"imu%05d.txt",usdfilenamenumber);
				printf("\n\rFile name created is %s",microsdname);
				StorageChoice = DONE;
			    }
			else {
			    usdfilenamemode = 'u';
		    	putfs(storagemess3);
				i=0;
					if(Uart1Channel == YES) {
						while(getcharnw1() != ':');
						while((inchar=getchar1()) != CR) {
							microsdname[i++] = inchar;
							putchar(inchar);
					 	    }
					    }
					else {
						while((inchar=getchar()) != CR) {
							microsdname[i++] = inchar;
							putchar(inchar);
						    }
					    }
					microsdname[i] = 0;
				StorageChoice = DONE;
				printf("\n\rFile name entered is %s",microsdname);
			    }
		    }
		// Serial Flash selected
		else if(inchar=='f') {
			storageselection = SERIALFLASH;
			StorageChoice = DONE;
		    }
		else {
			printf("\n\rNot a valid choice.\n\r");
			putfs(storagemess2);
		    }
	    }
	return 0;
}

// Check is uSD card installed - if not store to serial flash
// Serial flash has no file name
char StorageDeviceMenuTest()
{
	char inchar;
	int i=0;

	if(usdpresent==NO) {
		putfs(storagemess1);
		storageselection = SERIALFLASH;
		puts1("atd 0,0,33\r");
		puts1("ERROR: No Micro SD Card Detected!");
		putchar1('\r');
		WaitForOKERROR(VERBOSE,2000);
		//send2Master("ERROR: No Micro SD Card Detected!");
		return 0;
		} 
	else {
		storageselection = MICROSD;
		usdfilenamemode = 'u';
		while(getcharnw1() != ':');
		while((inchar=getchar1()) != CR) {
			microsdname[i++] = inchar;
			putchar(inchar);
		}
		microsdname[i] = 0;
		printf("\n\rFile name entered is %s",microsdname);		
	}
	return 0;
}

//  Time has started if day,hour, minute and second match or are greater than
unsigned char CheckStartTime()
{
	unsigned char gday,ghour,gminute,gsecond;

    cli();
		gday = days;
		ghour = hours;
		gminute = minutes;
		gsecond = seconds;
	sei();	

	//printf(" %d %d %d %d = %d %d %d %d\n\r",gdayghour,gminute,gsecond,daystart,hourstart,minutestart,secondstart);
	if((gday==daystart)&&(ghour==hourstart)&&(gminute==minutestart)&&(gsecond==secondstart)) {
	    printf("\n\rtime matched\n\r");
		timedstartmode = NO;
	    return YES;
		}
	return NO;
}

void DisplayOutputMenu(void)
{
	char inchar;

	printf("\n\rDisplay logging data [y|n]: ");
	inchar = getchar();
	putchar(inchar);
	if(inchar=='y') printoutput = YES;
	else printoutput = NO;
}

const char logmess1[] PROGMEM = "\n\rEnter timeout period (sec): ";

// This function manages the real time logging of sample data
// to the serial flash chip and uSD, starts/stops the logging and retrieves
// logged data
void LogDataToSerialFlash(void)
{
 	 char inchar,inchar2;
	 int i,delayperiod=0;

	 DataLoggerSignon();
	 DisplayCommandSummary();
	 putchar('>');
	 if(Uart1Channel == YES)
	 {
		if((inchar = getcharnw1()) == ':')
		{
			//while((inchar=getchar1())!='\r')	putchar(inchar);
			inchar=getchar1();
			putchar(inchar);
		}
	 }
	 else
	 {
	 	inchar = getcharr();
		putchar(inchar);
	 }
	 inchar2 = 'g';
	 //mseccountdown3 = delayperiod = 10000;  // exit after ten seconds of no activity 
	 while((inchar!='q')&&(inchar2!='b'))  {
	     //if(mseccountdown3==0)  break;
	     switch(inchar) {
	         case '?':
			     DataLoggerSignon();
		         break;
	         case 'j':
			     ThresholdModeMenu();
		         break;
	         case 'a':
			     mseccountdown3 = delayperiod;
			     if(armpb0==YES) {
				     printf("\n\rDisarm PB0");
				     armpb0 = NO;
					 SaveEepromPreferences();
					 }
				 else {
				     printf("\n\rArm PB0");
				     armpb0 = YES;
					 SetBit(PORTB,PINB0);  // Activate pullup
					 SaveEepromPreferences();
					 }
		         break;
	         case 't':
			 	 mseccountdown3 = delayperiod;  
			     putfs(loggermes145);
				 terminationmode = getchar();
				 putchar(terminationmode);
				 if(terminationmode==TIMEOUT) {
				 	 putfs(logmess1);
					 timeoutperiod = readlong();
					 SaveEepromPreferences();
					 }
		         break;
	         case 'p':
			     mseccountdown3 = delayperiod;  
				 printf("\n\rLogging start mode: (o-on powerup, n-not on powerup): ");
				 startloggingmode = getchar();
				 putchar(startloggingmode);
				 SaveEepromPreferences();
				 // disable it until next powerup
		         break;
	         case 'y':
			     mseccountdown3 = delayperiod;  
				 LogRestartMenu();
				 SaveEepromPreferences();
		         break;
	         case 'x':
			     mseccountdown3 = delayperiod;  
				 SensorRangeMenu();
				 SaveEepromPreferences();
		         break;
	         case 'v':
			     mseccountdown3 = delayperiod;  
				 DisplayOutputMenu();
				 SaveEepromPreferences();
		         break;
	         case 'u':
			     mseccountdown3 = delayperiod;  
				 StorageDeviceMenu();
				 SaveEepromPreferences();
		         break;
			case 'm':
			     mseccountdown3 = delayperiod;  
				 putfs(loggermes15);
				 minmax_mode = getchar();
				 putchar(minmax_mode);
				 SaveEepromPreferences();
		         break;
	         case 'w':
			     mseccountdown3 = delayperiod;  
				 SetupTimedStart();
				 SaveEepromPreferences();
		         break;
	         case 'e':
			     mseccountdown3 = delayperiod;  
			     GREEN_LED(ON);
			     FlashChipErase();
				 GREEN_LED(OFF);
		         break;
		     case 's':
			     mseccountdown3 = delayperiod;  
				 GREEN_LED(ON);
				 if(FlashBlankCheck()==YES) {
				     RED_LED(ON);
				     FlashChipErase();
					 RED_LED(OFF);
					 }
			     loggingmode = START;
				 if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
				 bufferidptr = 1;
				 pagecnt = 1;
				 inchar2 = 'b';
				 flashaddress = 0;
				 //putfs(loggermes16);
				 Init_BMP085_Readings();
		         break;
			 case 'd':
			     mseccountdown3 = delayperiod;  
				 SelectSensorsToLog();
				 SaveEepromPreferences();
				 printf("\n\r");
				 DisplayData2Log();
				 break;
		     case 'h':
			     mseccountdown3 = delayperiod;  
			     loggingmode = HALT;
				 GREEN_LED(OFF);
				 putfs(loggermes18);
		         break;
		     case 'i':
			     mseccountdown3 = delayperiod;  
			     putfs(loggermes193);
				 putchar(inchar=getchar());
				 if(inchar=='1') {
					 putfs(loggermes195);
				     strcpy(deviceidtext,"                    ");
					 for(i=0; i<20; i++)  {
					     inchar = getchar();
						 putchar(inchar);
						 if(inchar==CR) break;
						 else deviceidtext[i] = inchar;
						 }
					 SaveEepromPreferences();
					 printf("\n\rSensor ID 1 name changed to %s",deviceidtext);
					 }
				 else {
				     putfs(loggermes195);
				     strcpy(deviceidtext2,"                    ");
					 for(i=0; i<20; i++)  {
					     inchar = getchar();
						 putchar(inchar);
						 if(inchar==CR) break;
						 else deviceidtext2[i] = inchar;
						 }
					 SaveEepromPreferences();
					 printf("\n\rSensor ID 2 name changed to %s",deviceidtext2);
					 }
		         break;
			 case 'l':
			     mseccountdown3 = delayperiod;  
			     putfs(loggermes19);
				 localecho = YES;
				 scanf("%d",&defaultlograte);
				 localecho = NO;
				 getchar();
				 SaveEepromPreferences();
				 printf("\n\rLog rate changed to %d msec",defaultlograte);
				 DisplayMaxLogTime();
		         break;
		     case 'r':
			     mseccountdown3 = delayperiod;  
			     if(logoutputmode==VERBOSE)  putfs(loggermes21);
				 // Retreive logged data from each flash page
				 // until no more data left
				 RetreiveFlashData();
		         break;
	         case 'f':
			     mseccountdown3 = delayperiod;  
				 putfs(loggermes22);
				 logoutputmode = getchar();
				 putchar(logoutputmode);
				 SaveEepromPreferences();
		         break;
	         case 'c':
			     mseccountdown3 = delayperiod;  
			     putCRLF();
				 DisplayCommandSummary();
		         break;
		     }
		if(inchar2!='b') {
		    
			if(Uart1Channel == YES)
			{
				if((inchar = getcharnw1()) == ':')
				{
					//while((inchar=getchar1())!='\r')	putchar(inchar);
					inchar=getchar1();
					putchar(inchar);
					printf("\n\r>");
				}
			}
			else
			{
				 printf("\n\r>");
				 inchar = getcharr();
				 putchar(inchar);
			}
		  }
		}
	// disable until next powerup - place here so other changes don't wipe this out
	startloggingmode = NOTONPOWERUP;
}


