/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_Utilities.c	Version: 1.00
   
   Author:  SS	  					    Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19.
   
   This file contains routines that implement a set of basic utility 
   functions for the IMU64X0 Monitor.
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Utilities.c,v $
 * Revision 1.00  2013/02/22  SS
 *    2-wire interface, 8MHz clock, 19,200baud
 * Revision 0.86  2006/02/07  SS
 *    2-wire interface, 8MHz clock, 19,200baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Uart.h"
#include "IMU64X0_Extern.h"
#include "ff.h"
#include "diskio.h"

//*********************************************************
//*********************************************************
//      Utilities
//*********************************************************
//*********************************************************

// Calculate new future start time given delay in seconds
delay_time CalculateStartTime(long delaysec)
{
	long ltime;
	delay_time delaytime;

    ltime = ((long)days)*24*60*60 + ((long)hours)*60*60 + ((long)minutes)*60 + (long) seconds;
	ltime += delaysec;
	endtimelong = ltime;
	delaytime.day = ltime/(24*60*60);
	ltime -= delaytime.day*24*60*60;
	delaytime.hour = ltime/(60*60);
	ltime -= delaytime.hour*60*60;
	delaytime.minute = ltime/60;
	ltime -= delaytime.minute*60;
	delaytime.second = ltime;
	return delaytime;
}

void Delay(void)
{
	unsigned char a, b;

	for (a = 1; a; a++)
		for (b = 1; b; b++)
			;
}

void shortdelay(int cnt)
{
 	unsigned char inbyte;
	int i;
	
    for(i=0; i<cnt; i++)  {
	    inbyte = 0x55;
		}
}

void LongDelay(int cnt)
{
 	 while(cnt > 0) {
	 	 Delay();
		 cnt--;
		 }
}

void Delaymsec(int delayvalue)
{
 	 mseccountdown2 = delayvalue;
	 while(mseccountdown2) ;
}

void Delayseconds(int seconds)
{
 	 int i;
	 
 	 for(i=0; i<seconds; i++) Delaymsec(1000);
}

void RED_LED(unsigned char mode)
{
#ifdef IMU6410
 	if(mode==ON) ClrBit(PORTD,PIND5);	// low output to turn LED on
	else SetBit(PORTD,PIND5);	// low output to turn LED off
#else // IMU6420
 	if(mode==ON) ClrBit(PORTC,PINC4);	// low output to turn LED on
	else SetBit(PORTC,PINC4);	// low output to turn LED off
#endif
}

void GREEN_LED(unsigned char mode)
{
#ifdef IMU6410
 	 if(mode==ON) ClrBit(PORTD,PIND4);	
	 else SetBit(PORTD,PIND4);	

#else // IMU6420
 	 if(mode==ON) ClrBit(PORTC,PINC5);	
	 else SetBit(PORTC,PINC5);	
#endif
}

void BlinkGreen(int period)
{
 	 GREEN_LED(ON);
	 LongDelay(period);
	 GREEN_LED(OFF);
	 LongDelay(period);
}

void BlinkRed(int period)
{
 	 RED_LED(ON);
	 LongDelay(period);
	 RED_LED(OFF);
	 LongDelay(period);
}

void PrintHex(unsigned char inbyte)
{
     printf("%2x",inbyte);
}

void PrintWordHex(unsigned int inword)
{
 	 //printf("%4x",inword);
 	 addresstohex(inword);
	 printf("%s",hexaddress);
}

//  Access contents of AVR flash at specified address
unsigned char ReadAVRRAM(int address)
{
	unsigned char *intaddress;

	intaddress = address;
	  
	return *intaddress;                              
}

//  Access contents of AVR flash at specified address
unsigned char ReadAVRFlash(int address)
{
	unsigned char *intaddress;

	intaddress = address;
	  
	return *intaddress;                              
}

// ADC7 inputs the battery voltage divider network
// Internal voltage reference is either 2.56v or 1.1v
void BatteryVoltage(void)
{
     int batteryvoltage;
	 
 	 batteryvoltage = AnalogCh(7,INTERNALREF11V);
 	 batteryvoltage = AnalogCh(7,INTERNALREF11V);
 	 batteryvoltage = AnalogCh(7,INTERNALREF11V);
	 printf("\n\rBattery voltage = %d %1.3f volts\n\r",batteryvoltage,(float)batteryvoltage*0.00645);
// 	 batteryvoltage = AnalogCh(7,EXTERNALREF);
//	 printf("\n\rBattery voltage = %d %1.3f volts\n\r",batteryvoltage,(float)batteryvoltage*0.00375);
}

const char anamess1[] PROGMEM = "\n\rAnalog input summary\n\r";
const char anamess2[] PROGMEM = "Ch  average   min   max   diff\n\r";
const char anamess3[] PROGMEM = "  Y axis acceleration";
const char anamess4[] PROGMEM = "  Z axis acceleration";
const char anamess5[] PROGMEM = "  A Gyro";
const char anamess6[] PROGMEM = "  X axis acceleration";
const char anamess7[] PROGMEM = "  B Gyro";
const char anamess8[] PROGMEM = "  C Gyro";

// Read all the analog inputs and output min/max status
void MeasureAnalogInputs(void)
{
    int i,j,min,max,analogvalue,analogtotal;
	
	putfs(anamess1);
	putfs(anamess2);
	
	for(i=0; i<8; i++) {
		analogvalue = AnalogCh(i,EXTERNALREF);
		analogtotal = 0;
		min = max = analogvalue;
	    for(j=0; j<10; j++)  {
		    analogvalue = AnalogCh(i,EXTERNALREF);
			if(analogvalue < min)  min = analogvalue;
			if(analogvalue > max)  max = analogvalue;
			analogtotal += analogvalue;
			}
		printf("%4d     %4d    %4d    %4d    %4d",i,analogtotal/10,min,max,max-min);
		if(i==0) putfs(anamess3);
		else if(i==1) putfs(anamess4);
		else if(i==4) putfs(anamess5);
		else if(i==5) putfs(anamess6);
		else if(i==6) putfs(anamess7);
		else if(i==7) putfs(anamess8);
		putCRLF();
		}
}

//  Output sign on message loaded from Flash
void signon(void)
{
	//Delaymsec(5000);
#ifdef ATMEGA644TARGET
	putfs(signon1A);
#endif
#ifdef ATMEGA644PTARGET
	putfs(signon1B);
#endif
#ifdef ATMEGA1284PTARGET
	putfs(signon1C);
#endif
	putfs(signon2);

#ifdef	IMU6410
	CheckFlashTypes();
#endif

	printf("Device id1: %s  id2: %s\n\r",deviceidtext,deviceidtext2);

	putfs(signon3);
//	putfs(signon4);
	putfs(signon5);
	putfs(signon6);
	//putfs(signon62);
	putfs(signon65);
	putfs(signon66);
//	putfs(signon67);
	//putfs(signon7);
	putfs(signon8);
	putfs(signon10);
	putfs(signon83);
	putfs(signon85);
	putfs(signon9);
//	putfs(signon10);
}


//  Display AVR EEPROM memory
void DisplayEEPROM(int address,int numbytes)
{
 	 unsigned char i,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	     crcvalue = 0;
	     putcharr(':');
		 PrintHex(bytesonline);		 //  Output bytes on line
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 printf("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = EEPROM_read(i+newaddress);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
		 PrintHex((~crcvalue)+0x01);
	     printf("  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putcharr(':');
		 //  Output bytes on line
		 PrintHex(bytesonlastline);
		 crcvalue += bytesonlastline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     addresstohex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
	     printf("%s",hexaddress);
		 //  Output 00
		 printf("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = EEPROM_read(i+address+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
		 PrintHex((~crcvalue)+0x01);
	     printf("  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }

}

//  Display AVR Flash memory
void DisplayAVRFlash(int address,int numbytes)
{
 	 unsigned char i,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	 	 crcvalue = 0;
	     putcharr(':');
		 //  Output bytes on line
		 PrintHex(bytesonline);
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 printf("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     printf("  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putcharr(':');
		 //  Output bytes on line
		 PrintHex(bytesonlastline);
	     //  Output hex start address
	     PrintWordHex(address+linecnt*16);
		 //  Output 00
		 printf("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     printf("  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putcharr(ascii[i]);
			 else putcharr('.');
			 }
	     putCRLF();  
		 }

}

unsigned char hextobyte(unsigned char inbyte)
{
	 if((inbyte >= 'A')&(inbyte<='F')) return (inbyte - 'A' + 10);
	 if((inbyte >= 'a')&(inbyte<='f')) return (inbyte - 'a' + 10);
	 else return (inbyte - '0');
}

// Read in a hex string and convert to int or long
//  TBD - add long support for serial flash support
unsigned int  ScanHex(void)
{
 	 unsigned int  i=0,address;
 	 char len;
	 
	 address = 0;
	 putcharr((instrg[0]=getcharr()));
 	 while((instrg[i]!=CR)&&(instrg[i]!=' ')) {
	 	 i++;
	     putcharr((instrg[i]=getcharr()));
		 }
	 instrg[i] = 0;
	 len = strlen(instrg);
	 address = hextobyte(instrg[len-1]);
	 i = 1;
	 while(i<len) {
	     if(i==1) address += (hextobyte(instrg[len-i-1])*16);
	     if(i==2) address += (hextobyte(instrg[len-i-1])*256);
	     if(i==3) address += (hextobyte(instrg[len-i-1])*4096);
		 i++;
		 }
	 return address;
}

unsigned char GetByteHex(void)
{
 	 char strg[3];
	 
//	 putcharr((strg[0]=getchar()));
//	 putcharr((strg[1]=getchar()));
	 strg[0]=getcharr();
	 strg[1]=getcharr();
	 strg[2] = 0;
	 return converttohex(strg);
}

void DisplayMemory(void)
{
    unsigned char inchar;
	unsigned int address, length;
	
	inchar = getcharr();
	putcharr(inchar);
	putCRLF();
	putfs(displaymessage1);
	address = ScanHex();
	length = ScanHex();
	putCRLF();
	switch (inchar) {
	    case 'e': case 'E':
			DisplayEEPROM(address,length);
		    break;
		case 'a': case 'A':
		    DisplayAVRFlash(address,length);
		    break;
		//  In order to access Blackfin Flash the AVR must ask the Blackfin
		//  to release control of the SPI lines.  This is simulated in Rev A
		//  boards by resetting the Blackfin (which should release the SPI lines)
		//  On completion the AVR SPI lines are released.
		case 'f': case 'F':
//			Init_SPI_Master();
		    DisplaySerialFlash(address,length,READCONTINUOUS);	
//			Release_SPI();
		    break;
		case 'b': case 'B':
//			Init_SPI_Master();
		    DisplaySerialFlash(address,length,READBUFFER);			
//			Release_SPI();
		    break;
		default:
		    putfs(displaymessage33);
		    break;
	    }
}

// Check intelbinary CRC 
int checkcrc(unsigned char cycbyte)
{
	GetByteHex();
 	return 1;
}

// Convert Intel Hex ASCII string to binary
//
// :10 0100 00 0C948F0D0C941D0E0C941D0E0C941D0E 38
// :1001000000C941D0E0C941D0E0C944E590C941D0E38
// This routine decodes a line of Intel Hex.  Other 
// routines write the data to either AVR Flash, external Flash
// or AVR EEPROM
//
//   :02 0000 04 0000FA  - IntelHex32 super address record
// 
//  TBD - check crc byte - drop record if incorrect
int DecodeIntelHex(void)
{
 	 unsigned char inbyte;
	 int i;
	 
	 putcharr(':');
	 // Convert count of bytes on line
	 bytesonline = GetByteHex();
	 
	 // Convert address to binary - ignore filler byte
	 // Typical Intel Hex consists of 4 ASCII characters
	 inteladdress = GetByteHex()*256;
	 inteladdress += GetByteHex();
	 
	 // Read dump byte (2 characters)
	 intelrecordcommand = GetByteHex();
	 
	 if(intelrecordcommand==0x04) {
	     intelsuperaddress = GetByteHex()*256;
		 intelsuperaddress += GetByteHex();
		 }
	 else  {
	     // Convert all data to binary
	     for(i=0; i<bytesonline; i++) {
		     inbyte = GetByteHex();
		     intelbinary[i] = inbyte;
	 	     }
		 }
		 
	// Check CRC - todo
	if(checkcrc(inbyte)) return 1;
	else return -1;
}

//  Write decoded Intel hex to EEPROM
void ProgramEEPROM(void)
{
     unsigned char i;
	 
	 for(i=0; i<bytesonline; i++)  EEPROM_write(inteladdress+i,intelbinary[i]);
}

//  TBD - AVR Flash hold program so programming AVR flash may overwrite program
void ProgramAVRFlash(void)
{
 	 char inchar;
	 
 	 while((inchar=getcharr())!=CR) putcharr(inchar);
	 printf("\n-");
}

void CleanUpFlashState(void)
{
	if(flashdatainbuffer==YES)  {
	    inteladdress = oldinteladdress;  // end of file record destroyed intelad
	    ProgramBuffertoPageErase(1);
		WaitForFlashFree();
		}
	if(flashprogrammingactive==YES)  {
//		Release_SPI();
		flashprogrammingactive = NO;
		flashpageloaded = NO;
		flashpageboundarytraversed = NO;
		flashdatainbuffer = NO;
		}
}

//  Main memory programming selection function
//  Program EEPROM, AVR Flash or SPI Flash.
//  Three IntelHex32 Records 0x00, 0x01 and 0x04 are processed by this
//  routine.  0x00 is a data record, 0x01 is End of File record and 
//  0x04 is extended address record
//
//  TBD - no intel record time out - reset defaults if not intel 
//        records received after 3 seconds
void ProgramMemory(void)
{
 	DecodeIntelHex();
	switch (programmode) {
	    case LEEPROM:
			if(intelrecordcommand==0) ProgramEEPROM();
			break;
		case LAVRFLASH:
		    if(intelrecordcommand==0) ProgramAVRFlash();
			break;
		case LFLASH:
		    // Data record			
			if(intelrecordcommand==0) {
			    flashcountdown = 3000;  // 3 second no new Hex record timeout
			    //  See is SPI Master is active - set active if not
			    if(flashprogrammingactive==NO) {
//				    Init_SPI_Master();
					}
					
		        ProgramSerialFlash();		
				}
			// Extended address record
			else if(intelrecordcommand==0x04) ; // nothing to do
			
			// End of File record - reset all variables
			else if(intelrecordcommand==0x01) CleanUpFlashState();
			break;
		} 
}

//  Set default memory programming mode
//  Subsequent Intel Hex dumps will load to the specific memory
void LoadMemory(void)
{
    unsigned char inchar;
	
	inchar = getcharr();
	putcharr(inchar);
	switch (inchar) {
	    case 'e': case 'E':
			programmode = LEEPROM;
			putfs(loadmessage1);
		    break;
		case 'f': case 'F':
			programmode = LFLASH;
			putfs(loadmessage2);
		    break;
		case 'a': case 'A':
			programmode = LAVRFLASH;
		    putfs(loadmessage3);
		    break;
		default:
		   break;
	    }
}

//  Read selected analog channel - wait until it completes
//   TBD - background interrupt process
//   For internal reference:  1.1V if REFS= 10, 2.56V if REFS=11
int AnalogCh(unsigned char channel,char mode)
{
 	unsigned char low,high;

	if(mode==INTERNALREF11V)  ADMUX = (1<<REFS1)|channel;  // 1.1v ref
	else if(mode==INTERNALREF256V)  ADMUX = (1<<REFS1)|(1<<REFS0)|channel;  // 2.56v ref
	else ADMUX = (1<<REFS0)|channel;  // use AVCC

	ADCSRA = (1<<ADEN)|(1<<ADSC); // Enable ADC, start conversion, max clock
	shortdelay(2);
	while(!(ADCSRA&(1<<ADIF))) ; // loop until conversion completes
//	while((ADCSRA&(1<<ADSC))) ; // loop until conversion completes
	shortdelay(2);
	
	low = ADCL;
	high = ADCH;
	return (low + (high*256));
}

void DumpAnalogChs(char mode)
{
    int  i,analogvalue;
	
	printf("Analog Channels 0..7\n\r");
	if(mode==EXTERNALREF) {
	 	for(i=0; i<8; i++) {
		    analogvalue = AnalogCh(i,EXTERNALREF);
			printf("%d ",analogvalue);
			} 
	    printf("\n\r");
		}
	else if(mode==INTERNALREF11V) {
	 	for(i=0; i<8; i++) {
		    analogvalue = AnalogCh(i,INTERNALREF11V);
			printf("%d ",analogvalue);
			} 
		printf("\n\r");
		}
	else if(mode==INTERNALREF11V) { 
	 	for(i=0; i<8; i++) {
		    analogvalue = AnalogCh(i,INTERNALREF256V);
			printf("%d ",analogvalue);
			} 
		printf("\n\r"); 
		}
}

// Collect characters until CR, LF or space found
long readlong(void)
{
 	char incharbuf[14],inchar;
	int  intnum,i,endptr;
	long longvalue;
	
	intnum = 0;
	i = 0;
	// Skip preceeding white space
	while((inchar=getcharr())==' ') putchar(inchar);
	putchar(inchar);
	while((i<14)&&(inchar!=CR)&&(inchar!=LF)&&(inchar!=' ')) {
	    incharbuf[i++] = inchar;
		inchar = getcharr();
		putchar(inchar);
		}
	incharbuf[i] = 0;
	sscanf(incharbuf,"%ld",&longvalue);
	return longvalue;
}

// Collect characters until CR, LF or space found
int readint(void)
{
 	char incharbuf[10],inchar;
	int  intnum,i,endptr;
	long intvalue;
	
	intnum = 0;
	i = 0;
	// Skip preceeding white space
	while((inchar=getcharr())==' ') putchar(inchar);
	putchar(inchar);
	while((i<10)&&(inchar!=CR)&&(inchar!=LF)&&(inchar!=' ')) {
	    incharbuf[i++] = inchar;
		inchar = getcharr();
		putchar(inchar);
		}
	incharbuf[i] = 0;
	sscanf(incharbuf,"%d",&intvalue);
	return intnum;
}

char getint(void)
{
	char intvalue,inchar;
	
	inchar = getcharr();
	putcharr(inchar);
	if(inchar==CR) return 0x78;
	intvalue = (inchar - '0')*10;
	inchar = getcharr();
	putcharr(inchar);	
	intvalue = intvalue + (inchar-'0');
	return intvalue;
}


// Update the current day and time
void  getsettime(char mode)
{
    unsigned char instrg[3],inchar;
	
    cli();
	instrg[0] = hours;
	instrg[1] = minutes;
	instrg[2] = seconds;
	sei();	
	
	if(mode==VERBOSE) printf("\n\rSet time [hh:mm:ss]= %d:%d:%d :",
	  		instrg[0],instrg[1],instrg[2]);

	if((instrg[0]=getint())==0x78) return;
	inchar = getcharr(); if(mode==VERBOSE) putcharr(inchar);
	instrg[1] = getint();
	inchar = getcharr(); if(mode==VERBOSE) putcharr(inchar);
	instrg[2] = getint();
	inchar = getcharr(); if(mode==VERBOSE) putcharr(inchar);

    cli();
	hours = instrg[0];
	minutes = instrg[1];
	seconds = instrg[2];
	sei();	
}

// Update the current day and time
void  getsetdate(char mode)
{
    unsigned char instrg[4],inchar;	
	
	if(mode==VERBOSE) printf("\n\rSet date [mm dd yyyy]=%2d %2d %4d ",
	  		month,days,year);

	localecho = YES;
	scanf("%d %d %d",&month,&days,&year);
	localecho = NO;
	getchar();
}

void send2Master(char* msgbuf)
{
	Delaymsec(20);

	puts1("atd 0\r");
	//printf("atd 0\r");

	Delaymsec(20);

	puts1(msgbuf);
	//printf(msgbuf);

	Delaymsec(20);
	putchar1('\r');
	//putchar('\r');

	WaitForOKERROR(VERBOSE,2000);
	//printf("OK\n\r");
}

void set2Date(void)
{
    int cptr = 0;
	int setMonth, setDay, setYear;
	char instring[40];
	unsigned char incharSet;

	while((incharSet=getchar1())!='\r') {
		//putchar(inchar);
		instring[cptr++] = incharSet;
	}
	instring[cptr] = 0;
	//printf("%s", instring);

	sscanf(instring, " %2d/%2d/%4d",
	&setMonth, &setDay, &setYear);
	printf("Date is set to: %2d/%2d/%2d\n\r",
	setMonth, setDay, setYear);
	cli();
	month = setMonth;
	days = setDay;
	year = setYear;
	sei();
}

void get2Date(void)
{
	int getMonth, getDay, getYear;
	char msgbuf[40];
	cli();
	 getMonth = month;
	 getDay = days;
	 getYear = year;
	sei();
	printf("Internal device date is: %d/%d/%d\n\r",getMonth, getDay, getYear);
	//send to ZIGBEE
	sprintf(msgbuf, " %2d/%2d/%4d",getMonth, getDay, getYear);
	send2Master(msgbuf);
}

void set2Time(void)
{
	int cptr = 0;
	int setHour, setMinute, setSecond, setMillisecond;
	char instring[40];
	unsigned char incharSet;
	//printf("hit stop\n\r");
	//cptr = 0;
	while((incharSet=getchar1())!='\r') {
		//putchar(inchar);
		instring[cptr++] = incharSet;
	}
	instring[cptr] = 0;
	//printf("%s", instring);

	sscanf(instring, " %2d:%2d:%2d.%3d",
	&setHour, &setMinute, &setSecond, &setMillisecond);
	printf("Time is set to: %2d:%2d:%2d.%3d\n\r",
	setHour, setMinute, setSecond, setMillisecond);
	cli();
	hours = setHour;
	minutes = setMinute;
	seconds = setSecond;
	rtcclockcnt = setMillisecond/4;
	sei();
}

void get2Time(void)
{
	int getHour, getMinute, getSecond, getMillisecond;
	char msgbuf[40];
    cli();
	getHour = hours;
	getMinute = minutes;
	getSecond = seconds;
	getMillisecond = rtcclockcnt*4;
	sei();
	printf("Internal device time is: %d:%d:%d.%d\n\r",getHour, getMinute, getSecond, getMillisecond);
	//send to ZIGBEE
	sprintf(msgbuf, " %2d:%2d:%2d.%3d",getHour, getMinute, getSecond, getMillisecond);
	send2Master(msgbuf);
}

void initZigbeeSlave(void)
{
	int i, j;
	i = j = 0;
	int cptr = 0;
	char instring[50];
	char setNode[30];
	char setMac[15];
	unsigned char inchar;
	int Zig_WSRC = 0;
	int MacAddress = 0;
	char identifiers[45];

	while((inchar=getchar1())!='\r') {
		//putchar(inchar);
		instring[cptr++] = inchar;
	}
	instring[cptr] = 0;

	sscanf(instring, " %d,%d,%s", &Zig_WSRC, &MacAddress, identifiers);
	sprintf(setNode, "AT+WROLE=2 +WSRC=%d\r", Zig_WSRC);
	sprintf(setMac, "AT+GSN=%d\r", MacAddress);
	// AT+WLEAVE
	// ATX
	// AT+GSN= user-defined number for MacAddress	puts1("AT+GSN= \r");
	// AT+WPANID=1620
	// AT+WCHMASK=100000
	// AT+WROLE=2 +WSRC= user-defined number for NodeID
	// AT+WAUTONET=1 Z
	puts1("AT+WLEAVE\r");
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	puts1("ATX\r");
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	puts1(setMac);
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	puts1("AT+WPANID=1620\r");
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	puts1("AT+WCHMASK=100000\r");
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	puts1(setNode);
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	puts1("AT+WAUTONET=1 Z\r\r");
	WaitForOKERROR(VERBOSE,4000);
	while((inchar=getcharnw1())) {		
		Delaymsec(10);
		putchar(inchar);
	}
	printf("\n\rZigBee's new node and MAC Addr respectively are: %d, %d", Zig_WSRC, MacAddress);
	for(i=0; i < strlen(identifiers); i++) {
	if(identifiers[i] == ',')	break;
	deviceidtext[i] = identifiers[i];
	}
	deviceidtext[i] = 0;
	while(identifiers[i++] != '\r')		deviceidtext2[j++] = identifiers[i];
	deviceidtext2[j] = 0;
	printf("\n\rIMU Type: %s\n\rJoint: %s", deviceidtext, deviceidtext2);
	SaveEepromPreferences();
}

int zNodeFinder(void)
{
	int	i=0;
	int	j=0;
	int k=0;
	int cptr = 0;
	char instring[30];
	char inchar;
	int Zig_WSRC = 0;

	puts1("\n\r");
	puts1("AT+WSRC?\r");
	while((inchar=getchar1()))
	{
		instring[cptr++] = inchar;
		if(inchar == '\r')	k++;
		if(k == 3)	break;
	}
	instring[cptr] = 0;		
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	for(i=0; i < strlen(instring); i++)		if(instring[i] == ':')	break;
	i++;
	while((inchar = instring[i++]) != '\r')		instring[j++] = inchar;
	instring[j] = 0;
	sscanf(instring, "%04d", &Zig_WSRC);
	return(Zig_WSRC);
}

void checkZigbee(void)
{
	int statusMSG = 2;
	char inchar;

	while((inchar=getcharnw1()));  //putchar(inchar);
	puts1("AT\r");
	statusMSG = WaitForOKERROR(QUIET,1000);
	while((inchar=getcharnw1()));  //putchar(inchar);
	if(statusMSG==0)  printf("\rZigBee found node is: %d\n\r", zNodeFinder());
	else if(statusMSG==0)	printf("\rZigBee not found\n\r");
	else	printf("\rZigBee status unKnown\n\r");
}

void ZigbeeSurvey(void)
{
	char msgbuf[20];
	char inchar;

	//send to ZIGBEE
	puts1("AT\r");
	WaitForOKERROR(VERBOSE,1000);
	while((inchar=getcharnw1())) putchar(inchar);
	sprintf(msgbuf, "I am node: %04d", zNodeFinder());
	send2Master(msgbuf);
}

void UploadZigData(void)
{
	FATFS fs[1];         /* Work area (file system object) for logical drives */
    FIL fsrc;      /* file objects */
    BYTE buffer[78] = {'L','o','g',' ','S','t','a','r','t','s',':','\n','\r'};   /* file copy buffer */
    FRESULT res;         /* FatFs function common result code */
    UINT br;         /* File read/write count */
	char inchar;

    /* Register work area for each volume (Always succeeds regardless of disk status) */
//    f_mount(0, &fs[0]);

    // Open source file on the drive 1
    res = f_open(&fsrc, "zx.txt", FA_OPEN_EXISTING | FA_READ);
	if (res)	printf("\n\rNo Such File!\n\r");

	else {
	    // Copy source to destination
	    for (;;) {
	        send2Master(buffer);
			res = f_read(&fsrc, buffer, sizeof buffer, &br);     // Read a chunk of src file		
			if (res || br == 0) {
				puts1("atd 0\r");
				putchar1('z');
				putchar1('\r');
				WaitForOKERROR(VERBOSE,2000);
				break; // error or eof
			}
	    }
	    // Close open files
	    f_close(&fsrc);
		}


    // Unregister work area prior to discard it
 //   f_mount(0, NULL);
}

void getTime()
{
    cli();
	instrg[0] = hours;
	instrg[1] = minutes;
	instrg[2] = seconds;
	sei();	
 	printf("Internal device time is: %d:%d:%d\n\r",
	instrg[0],instrg[1],instrg[2]);
}

//  Simple test routine
void TestAVR(void)
{
 	 char i,j;
	 
 	 printf("\nTesting AVR Version 0.34\n");
//	 TestAVRRAM();
//	 TestAVREEPROM();
//	 TestAVRFLASH();
//	 TestBoardFLASH():
	 j = 0;
	 putcharr(' ');
	 for(i=0x20; i<0x7f; i++) {
	 	 putcharr(i);
		 if(j++ == 60)  {
		     j = 0;
			 printf("\n ");
			 }
		 }
	 printf("\n ");
	 DumpAnalogChs(EXTERNALREF);
}

//  TBD - turn interrupts off during write start
void  EEPROM_write(int uiAddress,unsigned char ucData)
{
	eeprom_write_byte(uiAddress, ucData); 
}

unsigned char EEPROM_read(int uiAddress)
{
	unsigned char inbyte;

	inbyte = eeprom_read_byte(uiAddress);

	// Return data from data register
	return inbyte;
}

//  Signature bytes are write to first five EEPROM locations 85 C2 1E 4F 11
void WriteEepromSignature(void)
{
    EEPROM_write(0,0x85);
    EEPROM_write(1,0xC2);
    EEPROM_write(2,0x1E);
    EEPROM_write(3,0x4F);
    EEPROM_write(4,0x11);
}

void WriteEepromUnsignedInt(int address,unsigned int ivalue)
{
    union u_tag {
	    unsigned int iivalue;
		unsigned char intchar[2];
		} u;
	
	u.iivalue = ivalue;
 	EEPROM_write(address++,u.intchar[0]);
 	EEPROM_write(address,u.intchar[1]);	
}

void WriteEepromInt(int address,int ivalue)
{
    union u_tag {
	    int iivalue;
		unsigned char intchar[2];
		} u;
	
	u.iivalue = ivalue;
 	EEPROM_write(address++,u.intchar[0]);
 	EEPROM_write(address,u.intchar[1]);	
}

void WriteEepromLong(int address,long ivalue)
{
    union u_tag {
	    long iivalue;
		unsigned char intchar[4];
		} u;
	
	u.iivalue = ivalue;
 	EEPROM_write(address++,u.intchar[0]);
 	EEPROM_write(address++,u.intchar[1]);	
 	EEPROM_write(address++,u.intchar[2]);	
 	EEPROM_write(address,u.intchar[3]);	
}

// Write program version - offset address 5
void WriteVersion(void)
{
    EEPROM_write(5,PRODUCTID);
	EEPROM_write(6,PROGRAMMAJORVER);
	EEPROM_write(7,PROGRAMMINORVER);
}

// Write copyright notice - offset address 8
void WriteCopyright(void)
{
    unsigned char ptr;
	
	for(ptr=100; ptr<(8+strlen(version)); ptr++)
          EEPROM_write(ptr,version[ptr-8]);
}

// Check 5 signature bytes in first five locations of EEPROM
//   Should be 85 C2 1E 4F 11
unsigned char CheckEepromSignature(void)
{
 	unsigned char status;
	
	status = 1;
	if(EEPROM_read(0)!=0x85) status = 0;
	if(EEPROM_read(1)!=0xC2) status = 0;
	if(EEPROM_read(2)!=0x1E) status = 0;
	if(EEPROM_read(3)!=0x4F) status = 0;
	if(EEPROM_read(4)!=0x11) status = 0;
	return status;
}

unsigned int ReadEepromUnsignedInt(int address)
{	
    union u_tag {
	    unsigned int ivalue;
		unsigned char intchar[2];
		} u;
	
	u.intchar[0] = EEPROM_read(address++);
	u.intchar[1] = EEPROM_read(address);
	return u.ivalue;	
}

int ReadEepromInt(int address)
{	
    union u_tag {
	    int ivalue;
		unsigned char intchar[2];
		} u;
	
	u.intchar[0] = EEPROM_read(address++);
	u.intchar[1] = EEPROM_read(address);
	return u.ivalue;	
}

long ReadEepromLong(int address)
{	
    union u_tag {
	    long ivalue;
		unsigned char intchar[4];
		} u;
	
	u.intchar[0] = EEPROM_read(address++);
	u.intchar[1] = EEPROM_read(address++);
	u.intchar[2] = EEPROM_read(address++);
	u.intchar[3] = EEPROM_read(address);
	return u.ivalue;	
}

void  WriteEepromFloat(int address,float fvalue)
{
    union u_tag {
	    float ffvalue;
		unsigned char floatchar[4];
		} u;
	
	u.ffvalue = fvalue;
 	EEPROM_write(address++,u.floatchar[0]);
 	EEPROM_write(address++,u.floatchar[1]);
 	EEPROM_write(address++,u.floatchar[2]);
 	EEPROM_write(address,u.floatchar[3]);	
}

float ReadEepromFloat(int address)
{	
    union u_tag {
	    float fvalue;
		unsigned char floatchar[4];
		} u;
	
	u.floatchar[0] = EEPROM_read(address++);
	u.floatchar[1] = EEPROM_read(address++);
	u.floatchar[2] = EEPROM_read(address++);
	u.floatchar[3] = EEPROM_read(address);
	return u.fvalue;	
}

//  Load all the Motor Controller Setup preferences from EEPROM
void LoadEepromPreferences(void)
{
    int addressptr,i;
	
	// Leave space for dad variables.
	addressptr = EEPROMBASEADDRESS;
	
	// Set esc mode
	escmode = EEPROM_read(addressptr++);

	// I2C Configuration
	i2cmode = EEPROM_read(addressptr++);
	TWI_slaveAddress = EEPROM_read(addressptr++);
	TWI_masterAddress = EEPROM_read(addressptr++);
	
	// General Setup parameters
	defaultlograte = ReadEepromInt(addressptr);  addressptr += 2;
	armpb0 = EEPROM_read(addressptr++);
	if(armpb0==YES) SetBit(PORTB,PINB0);  // Activate pull up
	data2log = EEPROM_read(addressptr++);
	defaultaccelerometerrange = EEPROM_read(addressptr++);
	defaultgyrorange = EEPROM_read(addressptr++);
	defaultmagnetometerrange = EEPROM_read(addressptr++);
	logoutputmode = EEPROM_read(addressptr++);
	minmax_mode = EEPROM_read(addressptr++);

	timedstartmode = EEPROM_read(addressptr++);

	terminationmode = EEPROM_read(addressptr++);
	timeoutperiod = ReadEepromLong(addressptr);  addressptr += 4;
	
	// Data Logger parameters
	storageselection = EEPROM_read(addressptr++);
	for(i=0; i<12; i++) microsdname[i] = EEPROM_read(addressptr++);
	thresholdpercent = ReadEepromFloat(addressptr); addressptr += 4;
	for(i=0; i<12; i++) thresholdcheck[i] = EEPROM_read(addressptr++);
	startloggingmode = EEPROM_read(addressptr++);
	usdfilenamemode = EEPROM_read(addressptr++);

	usdfilenamenumberoffset = addressptr;  // capture this offset so logging
										   // code doesn't loose position
	usdfilenamenumber = ReadEepromInt(addressptr);  addressptr += 2;

//	printf("usdfilenamenumber=%d  address=%d\n\r",usdfilenamenumber,addressptr);

	delaystarttimesec = ReadEepromInt(addressptr);  addressptr += 2;
	logrestart = EEPROM_read(addressptr++);
	logrestartmax = ReadEepromInt(addressptr);  addressptr += 2;

	logrestartcntoffset = addressptr;  // capture this parameter offset so logging
									   // code doesn't use wrong ptr
	logrestartcnt = ReadEepromInt(addressptr);  addressptr += 2;

	// Date parameters
	days = EEPROM_read(addressptr++);
	month = EEPROM_read(addressptr++);
	year = ReadEepromInt(addressptr); addressptr += 2;
		
	// Device id text message
	for(i=0; i<20; i++) deviceidtext[i] = EEPROM_read(addressptr++);
	for(i=0; i<20; i++) deviceidtext2[i] = EEPROM_read(addressptr++);

	printoutput = EEPROM_read(addressptr++);
}

// Save controller configuration information to EEPROM
void SaveEepromPreferences(void)
{
    int addressptr,i;

	// Leave space for dad variables.
	addressptr = EEPROMBASEADDRESS;

	// Set esc mode
	EEPROM_write(addressptr++,escmode);

	// I2C Configuration
	EEPROM_write(addressptr++,i2cmode);
	EEPROM_write(addressptr++,TWI_slaveAddress);
	EEPROM_write(addressptr++,TWI_masterAddress);

	// General Setup parameters
	WriteEepromInt(addressptr,defaultlograte); addressptr += 2;
	EEPROM_write(addressptr++,armpb0);
	EEPROM_write(addressptr++,data2log);
	EEPROM_write(addressptr++,defaultaccelerometerrange);
	EEPROM_write(addressptr++,defaultgyrorange);
	EEPROM_write(addressptr++,defaultmagnetometerrange);
	EEPROM_write(addressptr++,logoutputmode);
	EEPROM_write(addressptr++,minmax_mode);

	EEPROM_write(addressptr++,timedstartmode);

	EEPROM_write(addressptr++,terminationmode);
	WriteEepromLong(addressptr,timeoutperiod); addressptr += 4;	
	
	// Data Logger parameters
	EEPROM_write(addressptr++,storageselection);
	for(i=0; i<12; i++) EEPROM_write(addressptr++,microsdname[i]);
	WriteEepromFloat(addressptr,thresholdpercent); addressptr += 4;
	for(i=0; i<12; i++) EEPROM_write(addressptr++,thresholdcheck[i]);
	EEPROM_write(addressptr++,startloggingmode);
	EEPROM_write(addressptr++,usdfilenamemode);

	//printf("write usd num=%d\n\r",addressptr);
	usdfilenamenumberoffset = addressptr;  // capture this offset so logging
										   // code doesn't loose position
	WriteEepromInt(addressptr,usdfilenamenumber); addressptr += 2;

	WriteEepromInt(addressptr,delaystarttimesec); addressptr += 2;	
	EEPROM_write(addressptr++,logrestart);
	WriteEepromInt(addressptr,logrestartmax); addressptr += 2;
//	printf(" EEPROM address = %d\n\r",addressptr);
	
	logrestartcntoffset = addressptr;  // capture this parameter offset so logging
									   // code doesn't use wrong ptr
	WriteEepromInt(addressptr,logrestartcnt); addressptr += 2;	
//	printf("\n\rEEPROM logrestartcnt=%d\n\r",logrestartcnt);

	// Date parameters
	EEPROM_write(addressptr++,days);
	EEPROM_write(addressptr++,month);
	WriteEepromInt(addressptr,year); addressptr += 2;	


	// Store the device id text messages
	for(i=0; i<20; i++) EEPROM_write(addressptr++,deviceidtext[i]);
	for(i=0; i<20; i++) EEPROM_write(addressptr++,deviceidtext2[i]);

	EEPROM_write(addressptr++,printoutput);
}

//  Print all the Motor Controller Setup preferences from EEPROM
void PrintEepromPreferences(void)
{
    int addressptr,i;
	
	// Leave space for dad variables.
	addressptr = EEPROMBASEADDRESS;
	printf("\n\rStart EEPROM address = %d\n\r",addressptr);
	
	// Set esc mode
	escmode = EEPROM_read(addressptr++);
	printf("escmode = %x\n\r",escmode);

	// I2C Configuration
	i2cmode = EEPROM_read(addressptr++);
	TWI_slaveAddress = EEPROM_read(addressptr++);
	TWI_masterAddress = EEPROM_read(addressptr++);
	printf("i2cmode=%x %c  slaveid = %x  masterid = %x\n\r",i2cmode,i2cmode,
						TWI_slaveAddress,TWI_masterAddress);
	
	// General Setup parameters
	defaultlograte = ReadEepromInt(addressptr);  addressptr += 2;
	printf("defaultlograte = %d msec\n\r",defaultlograte);

	armpb0 = EEPROM_read(addressptr++);
	printf("armpb0 = %x\n\r",armpb0);

	if(armpb0==YES) SetBit(PORTB,PINB0);  // Activate pull up
	data2log = EEPROM_read(addressptr++);

	defaultaccelerometerrange = EEPROM_read(addressptr++);
	printf("defaultaccelerometerrange = %c\n\r",defaultaccelerometerrange);

	defaultgyrorange = EEPROM_read(addressptr++);
	printf("defaultgyrorange = %c\n\r",defaultgyrorange);

	defaultmagnetometerrange = EEPROM_read(addressptr++);
	printf("defaultmagnetometerrange = %x\n\r",defaultmagnetometerrange);

	logoutputmode = EEPROM_read(addressptr++);
	printf("logoutputmode = %c\n\r",logoutputmode);

	minmax_mode = EEPROM_read(addressptr++);
	printf("minmax_mode = %c\n\r",minmax_mode);

	timedstartmode = EEPROM_read(addressptr++);
	printf("timedstartmode = %d\n\r",timedstartmode);

	terminationmode = EEPROM_read(addressptr++);
	printf("terminationmode = %c\n\r",terminationmode);

	timeoutperiod = ReadEepromLong(addressptr);  addressptr += 4;
	printf("timeoutperiod = %ld seconds\n\r",timeoutperiod);

	// Data Logger parameters
	storageselection = EEPROM_read(addressptr++);
	printf("storageselection = %c\n\r",storageselection);

	for(i=0; i<12; i++) microsdname[i] = EEPROM_read(addressptr++);
	printf("microsdname = %s\n\r",microsdname);

	thresholdpercent = ReadEepromFloat(addressptr); addressptr += 4;
	printf("thresholdpercent = %f\n\r",thresholdpercent);

	printf("thresholdcheck = ");
	for(i=0; i<12; i++) {
	    thresholdcheck[i] = EEPROM_read(addressptr++);
		printf("%c ",thresholdcheck[i]);
		}
	printf("\n\r");

	startloggingmode = EEPROM_read(addressptr++);
	printf("startloggingmode = %c\n\r",startloggingmode);

	usdfilenamemode = EEPROM_read(addressptr++);
	printf("usdfilenamemode = %c\n\r",usdfilenamemode);

	//printf("usd num=%d\n\r",addressptr);
	usdfilenamenumber = ReadEepromInt(addressptr);  addressptr += 2;
	printf("usdfilenamenumber = %d\n\r",usdfilenamenumber);

	delaystarttimesec = ReadEepromInt(addressptr);  addressptr += 2;
	printf("delaystarttimesec = %d\n\r",delaystarttimesec);

	logrestart = EEPROM_read(addressptr++);
	printf("logrestart = %x %c\n\r",logrestart,logrestart);

	logrestartmax = ReadEepromInt(addressptr);  addressptr += 2;
	printf("logrestartmax = %d\n\r",logrestartmax);

	//printf("log restart cnt=%d\n\r",addressptr);
	logrestartcnt = ReadEepromInt(addressptr);  addressptr += 2;
	printf("logrestartcnt = %d\n\r",logrestartcnt);

	// Date parameters
	days = EEPROM_read(addressptr++);
	month = EEPROM_read(addressptr++);
	year = ReadEepromInt(addressptr); addressptr += 2;
	printf("day month year = %d %d %d\n\r",days,month,year);
		
	// Device id text message
	for(i=0; i<20; i++) deviceidtext[i] = EEPROM_read(addressptr++);
	printf("deviceidtext = %s\n\r",deviceidtext);
	for(i=0; i<20; i++) deviceidtext2[i] = EEPROM_read(addressptr++);
	printf("deviceidtext2 = %s\n\r",deviceidtext2);

	printoutput = EEPROM_read(addressptr++);
	printf("printoutput = %x\n\r",printoutput);
}

void SetDefaultEepromPreferences(void)
{
	int i;

	// ESC mode setting - if active ignore all commands until ESC received
	escmode = OFF;
	
    // IMU6410 System logging parameters
	defaultlograte = 10;
	defaultaccelerometerrange = G16MODE;
	defaultgyrorange = L250DPS;
	defaultmagnetometerrange = HMC130GA;
	logoutputmode = PROCESSED;
	minmax_mode = DISABLED;
	armpb0 = NO;
	timedstartmode = NO;
	terminationmode = HALT;
	thresholdmode = INACTIVE;
	thresholdpercent = 10.0;
	for(i=0; i<12; i++) thresholdcheck[i] = 'y';
	timeoutperiod = 140;  // seconds
	data2log = 0x0f;  // Log accel, gyro, mag and baro
	dataformat = PROCESSED;
	printoutput = YES;

#ifdef IMU6410
	strcpy(deviceidtext,"IMU6410 V1.00 A");
	strcpy(deviceidtext2,"Right Elbow");
	strcpy(microsdname,"IMU6410.txt ");
#else
	strcpy(deviceidtext,"IMU6420 V1.00 A");
	strcpy(deviceidtext2,"Right Elbow");
	strcpy(microsdname,"IMU6420.txt ");
#endif

	deviceid = SLED;

	storageselection = SERIALFLASH;

	startloggingmode = NOTONPOWERUP;
	usdfilenamemode = UNIQUE;

	logrestart = NO;
	logrestartcnt = 0;
	logrestartmax = 32000;
	usdfilenamenumber = 1;

	days = 0;
	month = 2;
	year = 2013;

	// If DAD mode active don't output signon message and 
	// DAD commands immediately.  DAD mode is activated by
	// sending ESC d
	dadmode = INACTIVE;
	loggingmode = HALT;
	pagemode = NEW;

	// I2C Mode Settings
	i2cmode = ACTIVE;
	TWI_slaveAddress = TWI_SLAVEADDRESS;
	TWI_masterAddress = TWI_MASTERADDRESS;
}


