//**************************************************************
// ****** FUNCTIONS FOR SD RAW DATA TRANSFER *******
//**************************************************************
//Controller		: ATmega32 (Clock: 8 Mhz-internal)
//Compiler			: AVR-GCC (winAVR with AVRStudio-4)
//Project Version	: DL_1.0
//Author			: CC Dharmani, Chennai (India)
//			  		  www.dharmanitech.com
//Date				: 10 May 2011
//**************************************************************

//Link to the Post: http://www.dharmanitech.com/2009/01/sd-card-interfacing-with-atmega8-fat32.html

// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>
//#include <avr/io.h>

//#include "SPI_routines.h"
#include "SD_routines.h"
#include "RTC_routines.h"
#include "FAT32.h"
#include "stdio.h"
//#include "usart_driver.h"

#include "IMU6410_Uart.h"
#include "IMU6410_Defines.h"
#include "IMU6410_Extern.h"

//extern void PutChar(unsigned char inchar);
//extern unsigned char GetChar(void);
//extern void Delayusec(unsigned int delay);

// Setup printf support definitions
//extern static int uart_putchar(char c, FILE *stream);

//extern static int uart_getchar(FILE *stream);
extern unsigned char localecho,convertlftocr;

// Commands for uSD SPI 
//extern void uSD_SPI_MasterTransmit(unsigned char inbyte);
//extern unsigned char uSD_SPI_MasterReceive(void);

unsigned char SPI_transmit(unsigned char inbyte)
{
	SPI_MasterTransmit(inbyte);
	//printf(" %x ",inbyte);
	//getchar();
	return 0;
}

unsigned char  bytesrx[400];
int maxcnt = 0;

unsigned char SPI_receive()
{
	unsigned char inbyte;

	inbyte = SPI_MasterReceive();
//	if(maxcnt<400) bytesrx[maxcnt++] = inbyte;
	return inbyte;
}

//******************************************************************
//Function	: to initialize the SD/SDHC card in SPI mode
//Arguments	: none
//return	: unsigned char; will be 0 if no error,
// 			  otherwise the response byte will be sent
//******************************************************************
unsigned char SD_init(void)
{
	unsigned char i, response, SD_version,inchar;
	unsigned int retry=0 ;

	for(i=0;i<10;i++)
        SPI_receive();   //80 clock pulses spent before sending the first command

	SD_CS_ASSERT;

	do	{
	   response = SD_sendCommand(GO_IDLE_STATE, 0); //send 'reset & go idle' command
	   retry++;
	   if(retry>0xFE) return 1;   //time out, card not detected
	   } while(response != 0x01);

	SD_CS_DEASSERT;
	SPI_transmit (0xff);
	SPI_transmit (0xff);

	retry = 0;

	SD_version = 2; //default set to SD compliance with ver2.x; 
					//this may change after checking the next command
	do	{
		response = SD_sendCommand(SEND_IF_COND,0x000001AA); //Check power supply status, mendatory for SDHC card
		retry++;

		if(retry>0xFE)  {
			  SD_version = 1;
			  cardType = 1;
			  return;
			  //break;
		   } //time out

		}while(response != 0x01);

	do	{
	    response = SD_sendCommand(APP_CMD,0); //CMD55, must be sent before sending any ACMD command
	    response = SD_sendCommand(SD_SEND_OP_COND,0x40000000); //ACMD41

	    retry++;
		shortdelay(10);
	    if(retry>0xfe)  {
		    return 2;  //time out, card initialization failed
	    } 

	}while(response != 0x00);

	retry = 0;
	SDHC_flag = 0;

	if (SD_version == 2)
	{ 
	   do
	   {
		 response = SD_sendCommand(READ_OCR,0);
		 retry++;
		 if(retry>0xfe) 
	     {
		  printf("\n\r");
		   cardType = 0;
		   break;
	     } //time out

	   }while(response != 0x00);

	   if(SDHC_flag == 1) cardType = 2;
	   else cardType = 3;
	}

	//SD_sendCommand(CRC_ON_OFF, OFF); //disable CRC; default - CRC disabled in SPI mode
	//SD_sendCommand(SET_BLOCK_LEN, 512); //set block size to 512; default size is 512


	return 0; //successful return
}

//******************************************************************
//Function	: to send a command to SD card
//Arguments	: unsigned char (8-bit command value)
// 			  & unsigned long (32-bit command argument)
//return	: unsigned char; response byte
//******************************************************************
unsigned char SD_sendCommand(unsigned char cmd, unsigned long arg)
{
    unsigned char response, retry=0, status;

	//SD card accepts byte address while SDHC accepts block address in multiples of 512
	//so, if it's SD card we need to convert block address into corresponding byte address by 
	//multipying it with 512. which is equivalent to shifting it left 9 times
	//following 'if' loop does that

	if(SDHC_flag == 0)		
	if(cmd == READ_SINGLE_BLOCK     ||
	   cmd == READ_MULTIPLE_BLOCKS  ||
	   cmd == WRITE_SINGLE_BLOCK    ||
	   cmd == WRITE_MULTIPLE_BLOCKS ||
	   cmd == ERASE_BLOCK_START_ADDR|| 
	   cmd == ERASE_BLOCK_END_ADDR ) 
	   {
	     arg = arg << 9;
	   }	   

	SD_CS_ASSERT;

	SPI_transmit(cmd | 0x40); // Send command, first two bits always '01'
	SPI_transmit((unsigned char)(arg>>24));
	SPI_transmit((unsigned char)(arg>>16));
	SPI_transmit((unsigned char)(arg>>8));
	SPI_transmit((unsigned char)(arg));

	if(cmd == SEND_IF_COND)	{ //it is compulsory to send correct CRC for CMD8 (CRC=0x87) & CMD0 (CRC=0x95)
	   SPI_transmit(0x87);    //for remaining commands, CRC is ignored in SPI mode
	   }
	else 
	  SPI_transmit(0x95); 

	while((response = SPI_receive()) == 0xff) //wait response
	   if(retry++ > 0xfe) break; //time out error 

	if((response == 0x00) && (cmd == 58))  //checking response of CMD58
	{
		status = SPI_receive() & 0x40;     //first byte of the OCR register (bit 31:24)
		if(status == 0x40) SDHC_flag = 1;  //we need it to verify SDHC card
		else SDHC_flag = 0;

		SPI_receive(); //remaining 3 bytes of the OCR register are ignored here
		SPI_receive(); //one can use these bytes to check power supply limits of SD
		SPI_receive(); 
	}

	SPI_receive(); //extra 8 CLK
	SD_CS_DEASSERT;

	return response; //return state
}

//*****************************************************************
//Function	: to erase specified no. of blocks of SD card
//Arguments	: none
//return	: unsigned char; will be 0 if no error,
// 			  otherwise the response byte will be sent
//*****************************************************************
unsigned char SD_erase (unsigned long startBlock, unsigned long totalBlocks)
{
	unsigned char response;

	response = SD_sendCommand(ERASE_BLOCK_START_ADDR, startBlock); //send starting block address
	if(response != 0x00) //check for SD status: 0x00 - OK (No flags set)
	  return response;

	response = SD_sendCommand(ERASE_BLOCK_END_ADDR,(startBlock + totalBlocks - 1)); //send end block address
	if(response != 0x00)
	  return response;

	response = SD_sendCommand(ERASE_SELECTED_BLOCKS, 0); //erase all selected blocks
	if(response != 0x00)
	  return response;

	return 0; //normal return
}

//******************************************************************
//Function	: to read a single block from SD card
//Arguments	: none
//return	: unsigned char; will be 0 if no error,
// 			  otherwise the response byte will be sent
//******************************************************************
unsigned char SD_readSingleBlock(unsigned long startBlock)
{
	unsigned char response;
	unsigned int i, retry=0;

	response = SD_sendCommand(READ_SINGLE_BLOCK, startBlock); //read a Block command
 
	if(response != 0x00) return response; //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	retry = 0;
	while(SPI_receive() != 0xfe) //wait for start block token 0xfe (0x11111110)
	  if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;} //return if time-out

	for(i=0; i<512; i++) //read 512 bytes
	  buffer[i] = SPI_receive();

	SPI_receive(); //receive incoming CRC (16-bit), CRC is ignored here
	SPI_receive();

	SPI_receive(); //extra 8 clock pulses
	SD_CS_DEASSERT;

	return 0;
}

//******************************************************************
//Function	: to write to a single block of SD card
//Arguments	: none
//return	: unsigned char; will be 0 if no error,
// 			  otherwise the response byte will be sent
//******************************************************************
unsigned char SD_writeSingleBlock(unsigned long startBlock)
{
	unsigned char response;
	unsigned int i, retry=0;

	response = SD_sendCommand(WRITE_SINGLE_BLOCK, startBlock); //write a Block command
  
	if(response != 0x00) return response; //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	SPI_transmit(0xfe);     //Send start block token 0xfe (0x11111110)

	for(i=0; i<512; i++)    //send 512 bytes data
	  SPI_transmit(buffer[i]);

	SPI_transmit(0xff);     //transmit dummy CRC (16-bit), CRC is ignored here
	SPI_transmit(0xff);

	response = SPI_receive();

	if( (response & 0x1f) != 0x05) //response= 0xXXX0AAA1 ; AAA='010' - data accepted
	{                              //AAA='101'-data rejected due to CRC error
	  SD_CS_DEASSERT;              //AAA='110'-data rejected due to write error
	  return response;
	}

	while(!SPI_receive()) //wait for SD card to complete writing and get idle
	if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;}

	SD_CS_DEASSERT;
	SPI_transmit(0xff);   //just spend 8 clock cycle delay before reasserting the CS line
	SD_CS_ASSERT;         //re-asserting the CS line to verify if card is still busy

	while(!SPI_receive()) //wait for SD card to complete writing and get idle
	   if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;}
	SD_CS_DEASSERT;

	return 0;
}


#ifndef FAT_TESTING_ONLY

//***************************************************************************
//Function	: to read multiple blocks from SD card & send every block to UART
//Arguments	: none
//return	: unsigned char; will be 0 if no error,
// 			  otherwise the response byte will be sent
//****************************************************************************
unsigned char SD_readMultipleBlock (unsigned long startBlock, unsigned long totalBlocks)
{
	unsigned char response;
	unsigned int i, retry=0;

	retry = 0;

	response = SD_sendCommand(READ_MULTIPLE_BLOCKS, startBlock); //write a Block command
  
	if(response != 0x00) return response; //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	while( totalBlocks )
	{
		  retry = 0;
		  while(SPI_receive() != 0xfe) //wait for start block token 0xfe (0x11111110)
		  if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;} //return if time-out

		  for(i=0; i<512; i++) //read 512 bytes
		    buffer[i] = SPI_receive();

		  SPI_receive(); //receive incoming CRC (16-bit), CRC is ignored here
		  SPI_receive();

		  SPI_receive(); //extra 8 cycles

		  printf("\n\r --------- \n\r"));

		  for(i=0; i<512; i++) //send the block to UART
		  {
		    if(buffer[i] == '~') break;
		    transmitByte ( buffer[i] );
		  }

		  printf("\n\r --------- \n\r"));
		  totalBlocks--;
		  }

	SD_sendCommand(STOP_TRANSMISSION, 0); //command to stop transmission
	SD_CS_DEASSERT;
	SPI_receive(); //extra 8 clock pulses

	return 0;
}

//***************************************************************************
//Function: to receive data from UART and write to multiple blocks of SD card
//Arguments: none
//return: unsigned char; will be 0 if no error,
// otherwise the response byte will be sent
//****************************************************************************
unsigned char SD_writeMultipleBlock(unsigned long startBlock, unsigned long totalBlocks)
{
	unsigned char response, data;
	unsigned int i, retry=0;
	unsigned long blockCounter=0, size;

	response = SD_sendCommand(WRITE_MULTIPLE_BLOCKS, startBlock); //write a Block command

	if(response != 0x00) return response; //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	printf("\n\r Enter text (End with ~): \n\r"));

	while( blockCounter < totalBlocks )
	{
	   i=0;
	   do
	   {
	     data = receiveByte();
	     if(data == 0x08)	//'Back Space' key pressed
		 { 
		   if(i != 0)
		   { 
		     transmitByte(data);
		     transmitByte(' '); 
		     transmitByte(data); 
		     i--; 
			 size--;
		   } 
		   continue;     
		 }
	     transmitByte(data);
	     buffer[i++] = data;
	     if(data == 0x0d)
	     {
	        transmitByte(0x0a);
	        buffer[i++] = 0x0a;
	     }
		 if(i == 512) break;
	   }while (data != '~');

	   printf("\n\r ---- \n\r"));

	   SPI_transmit(0xfc); //Send start block token 0xfc (0x11111100)

	   for(i=0; i<512; i++) //send 512 bytes data
	     SPI_transmit( buffer[i] );

	   SPI_transmit(0xff); //transmit dummy CRC (16-bit), CRC is ignored here
	   SPI_transmit(0xff);

	   response = SPI_receive();
	   if( (response & 0x1f) != 0x05) //response= 0xXXX0AAA1 ; AAA='010' - data accepted
	   {                              //AAA='101'-data rejected due to CRC error
	      SD_CS_DEASSERT;             //AAA='110'-data rejected due to write error
	      return response;
	   }

	   while(!SPI_receive()) //wait for SD card to complete writing and get idle
	     if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;}

	   SPI_receive(); //extra 8 bits
	   blockCounter++;
       }

	SPI_transmit(0xfd); //send 'stop transmission token'

	retry = 0;

	while(!SPI_receive()) //wait for SD card to complete writing and get idle
	   if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;}

	SD_CS_DEASSERT;
	SPI_transmit(0xff); //just spend 8 clock cycle delay before reasserting the CS signal
	SD_CS_ASSERT; //re assertion of the CS signal is required to verify if card is still busy

	while(!SPI_receive()) //wait for SD card to complete writing and get idle
	   if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;}
	SD_CS_DEASSERT;

	return 0;
}
//*********************************************

#endif

//******** END ****** www.dharmanitech.com *****


void uSD_Init()
{
	// Setup Port B
	//   PB0 - NC
	//   PB1 - ADXL345 CS
	//   PB2 - L3G4200 CS
	//   PB3 - Chip Select for uSD on IMU6410
	//   PB4 - SS
	//DDRB |= 0x08;
	//PORTB |= 0x08;

	Init_SPI_Master();
}

extern	struct BS_Structure *bpb; //mapping the buffer onto the structure

void Display_microSD_BootRecord()
{
//Structure to access boot sector data

/*unsigned char jumpBoot[3]; //default: 0x009000EB
unsigned char OEMName[8];
unsigned int bytesPerSector; //deafault: 512
unsigned char sectorPerCluster;
unsigned int reservedSectorCount;
unsigned char numberofFATs;
unsigned int rootEntryCount;
unsigned int totalSectors_F16; //must be 0 for FAT32
unsigned char mediaType;
unsigned int FATsize_F16; //must be 0 for FAT32
unsigned int sectorsPerTrack;
unsigned int numberofHeads;
unsigned long hiddenSectors;
unsigned long totalSectors_F32;
unsigned long FATsize_F32; //count of sectors occupied by one FAT
unsigned int extFlags;
unsigned int FSversion; //0x0000 (defines version 0.0)
unsigned long rootCluster; //first cluster of root directory (=2)
unsigned int FSinfo; //sector number of FSinfo structure (=1)
unsigned int BackupBootSector;
unsigned char reserved[12];
unsigned char driveNumber;
unsigned char reserved1;
unsigned char bootSignature;
unsigned long volumeID;
unsigned char volumeLabel[11]; //"NO NAME "
unsigned char fileSystemType[8]; //"FAT32"
unsigned char bootData[420];
unsigned int bootEndSignature; //0xaa55
*/
	printf("Structure of microSD Boot Record\n\r");
	printf("OEMName=%s\n\r",bpb->OEMName);
	printf("Sector size: %d bytes\n\r",bytesPerSector);
	printf("FAT sectors: %ld\n\r",totaldataSectors);
	printf("uSD total size: %ld\n\r",bytesPerSector*totaldataSectors);

}

extern unsigned char bytesrx[];
extern int maxcnt;

// FAT32 File System Test
int FAT32_FileSystem_Test()
{
	char inchar;
    unsigned char option, error, i, j, data, channel;
    unsigned char fileName[13];

	printf("\n\r****************************************************");
	printf("\n\r           IMU6410 microSD Datalogger  ");
	printf("\n\r****************************************************\n\r");

	uSD_Init();

	cardType = 0;

	for (i=0; i<10; i++) {
		error = SD_init();
		if(!error) break;
		}

	if(error) {
		if(error == 1) printf("SD card not detected..");
		if(error == 2) printf("Card Initialization failed..");
		}

	switch (cardType){
	  	case 1:
			printf("\n\rStandard Capacity Card (Ver 1.x) Detected!");
	  		break;
	  	case 2:
		    printf("\n\rHigh Capacity Card Detected!");
	  		break;
	  	case 3:
		    printf("\n\rStandard Capacity Card (Ver 2.x) Detected!");
	  		break;
	  	default:
		    printf("\n\rUnknown SD Card Detected!");
	  		break; 
	   }

	error = getBootSectorData (); //read boot sector and keep necessary data in global variables
	if(error) {
		printf("\n\rFAT32 not found!");  //FAT32 incompatible drive
		}

//	SPI_HIGH_SPEED;	//SCK - 4 MHz
	Delaymsec(1);   	//some delay for settling new spi speed

//	printf("\n\rmaxcnt=%d\n\r",maxcnt);
//	for(ptr=0; ptr<maxcnt; ptr++) printf("%x ",bytesrx[ptr]);
	if(error) return 0;

	//For displaying menu on hyper terminal, the key (psh-button) must be kept pressed while 
    //powering ON or while reset. If key is not kept pressed, the program will not display menu and it will 
    //simply wait for start recording command (i.e. pressing of key afterwards)

	while(1)  {
	  	printf("\n\r\n\r> q : Exit the Menu");
	  	printf("\n\r> 1 : Display current Date/Time");
	  	printf("\n\r> 2 : Update Date");
	  	printf("\n\r> 3 : Update Time");
	  	printf("\n\r> 4 : Get file list");
	  	printf("\n\r> 5 : Read File");
	  	printf("\n\r> 6 : Delete File");
	  	printf("\n\r> 7 : Create test file");
	  	printf("\n\r> 8 : uSD Boot Record Contents");
		printf("\n\r> 9 : Erase uSD");

	  	printf("\n\r\n\r> Enter the option:");
	  	option = GetChar();
	  	PutChar(option);


		switch (option)	{
			case 'q':
				printf("\n\rNormal operation started..");
			  	return 0;

			case '1':
				RTC_displayDate();
			  	RTC_displayTime();
			  	break;

			case '2':
				RTC_updateDate();
			  	break;

			case '3':
				RTC_updateTime();
		      	break;

			case '4':
				printf("\n\r");
	  		  	findFiles(GET_LIST,0);
	          	break;

			case '5':
			case '6':
				printf("\n\rEnter file name: ");
	          	for(i=0; i<13; i++)
				fileName[i] = 0x00;   //clearing any previously stored file name
	          	i=0;
	          	while(1) {
	            	data = GetChar();
	            	if(data == 0x0d) break;  //'ENTER' key pressed
					if(data == 0x08)	//'Back Space' key pressed
		 				{ 
		   					if(i != 0)
		   					{ 
		     					PutChar(data);
								PutChar(' '); 
		     					PutChar(data); 
		     					i--; 
		   					} 
		   					continue;     
		 				}
						if(data <0x20 || data > 0x7e) continue;  //check for valid English text character
						PutChar(data);
	            		fileName[i++] = data;
	            		if(i==13){printf(" file name too long.."); break;}
	          		}
	          		if(i>12) break;
       
			  		if(option == '5') 
					{ 
						error = readFile( READ, fileName);
						if(error == 1) printf("\n\rFile does not exist..");
					}
			  		if(option == '6') deleteFile(fileName);
	          		break;

			case '7':
				printf("\n\rCreate large data record file called foo5.txt");
				strcpy(fileName,"foo5.txt");
				strcpy(dataString,"12:23:34 34.234 0.94 0.85 0.98 12.34 23.23 24.24 0.123 0.234 0.345\n\r");
				printf("\n\r");
				printtime();
				printf("\n\r");
				for(i=0; i<2; i++)  {
				    strcpy(fileName,"foo6.txt");
				    strcpy(dataString,"12:23:34 34.234 0.94 0.85 0.98 12.34 23.23 24.24 0.123 0.234 0.345\n\r");
					//Delaymsec(50);
					error = writeFile(fileName);
					if(getcharnw()=='e') break;
					}
				printf("\n\r");
				printtime();
				printf("\n\r");
	          	break;

			case '8':
				printf("\n\r");
	  		  	Display_microSD_BootRecord();
	          	break;

			case '9':
				printf("\n\rErase uSD - TBD");
	  		  	
	          	break;

	    	default:printf("\n\r\n\r Invalid option!\n\r");
		}
	  }

  
  while(1)  { 

	inchar = GetChar();

	while(1)  {

//		error = RTC_getDate();

		j=0;
		for(i=0; i<8; i++) {
			fileName[i] = date[j++];
			if(j==2 || j==5) j++;	//excluding the '/' character from date in the fileName
		    }

		fileName[8] = '.';
		fileName[9] = 'C';
		fileName[10] = 'S';
		fileName[11] = 'V';

		error = RTC_getTime();
								
		//From here onwards, gather data by appending strings in dataString
		//dataString is declared in FAT32.h
		//make sure dataString doesn't exceed its MAX_STRING_SIZE, defined in FAT32.h
		//Also, end the data string with '\r' & '\n' characters to maintain CSV format

		for(i=0; i<10; i++) dataString[i] = date[i];
		dataString[i++] = ',';

		for(j=0;j<8; j++)  dataString[i++] = time[j];
		dataString[i++] = ',';
		
	//	readTemperature(0); //read temperature from adc channel-0
	//	for(j=0;j<7; j++)  dataString[i++] = temperature[j];
		

		for(channel=1; channel<8; channel++) { //read voltages from ADC channel 1 to 7
			dataString[i++] = ',';
//			readVoltage(channel); 
//			for(j=0;j<7; j++)  dataString[i++] = voltage[j];
		    }

		dataString[i++] = '\r';
		dataString[i] = '\n';    //always end the string with these two characters,
								 //before calling the writeFile function

		error = writeFile(fileName);

		}

	}

}
